@@ -39,8 +39,6 @@
 #include "fcitximcontext.h"
 #include "fcitx-gclient/fcitxclient.h"
 
-#define LOG_LEVEL DEBUG
-
 #if !GTK_CHECK_VERSION (2, 91, 0)
 # define DEPRECATED_GDK_KEYSYMS 1
 #endif
@@ -60,17 +58,15 @@ struct _FcitxIMContext {
     guint32 time;
     gboolean use_preedit;
     gboolean is_inpreedit;
-    char* preedit_string;
+    gchar* preedit_string;
+    gchar* surrounding_text;
     int cursor_pos;
     FcitxCapacityFlags capacity;
     PangoAttrList* attrlist;
+    gint last_cursor_pos;
+    gint last_anchor_pos;
 };
 
-typedef struct _ProcessKeyStruct {
-    FcitxIMContext* context;
-    GdkEventKey* event;
-} ProcessKeyStruct;
-
 struct _FcitxIMContextClass {
     GtkIMContextClass parent;
     /* klass members */
@@ -78,7 +74,7 @@ struct _FcitxIMContextClass {
 
 /* functions prototype */
 static void     fcitx_im_context_class_init(FcitxIMContextClass   *klass);
-static void     fcitx_im_context_class_fini (FcitxIMContextClass *klass);
+static void     fcitx_im_context_class_fini(FcitxIMContextClass *klass);
 static void     fcitx_im_context_init(FcitxIMContext        *im_context);
 static void     fcitx_im_context_finalize(GObject               *obj);
 static void     fcitx_im_context_set_client_window(GtkIMContext          *context,
@@ -93,15 +89,11 @@ static void     fcitx_im_context_set_cursor_location(GtkIMContext          *cont
 static void     fcitx_im_context_set_use_preedit(GtkIMContext          *context,
         gboolean               use_preedit);
 static void
-fcitx_im_context_set_surrounding (GtkIMContext *context,
-                                  const gchar *text,
-                                  gint len,
-                                  gint cursor_index);
-static void     fcitx_im_context_get_preedit_string(GtkIMContext          *context,
-        gchar                **str,
-        PangoAttrList        **attrs,
-        gint                  *cursor_pos);
-
+fcitx_im_context_set_surrounding(GtkIMContext *context, const gchar *text,
+                                 gint len, gint cursor_index);
+static void
+fcitx_im_context_get_preedit_string(GtkIMContext *context, gchar **str,
+                                    PangoAttrList **attrs, gint *cursor_pos);
 
 static gboolean
 _set_cursor_location_internal(FcitxIMContext *fcitxcontext);
@@ -145,6 +137,17 @@ _fcitx_im_context_process_key_cb(GObject* source_object, GAsyncResult* res, gpoi
 static void
 _fcitx_im_context_set_capacity(FcitxIMContext* fcitxcontext, gboolean force);
 
+#if GTK_CHECK_VERSION(3, 6, 0)
+
+static void
+_fcitx_im_context_input_hints_changed_cb(GObject *gobject, GParamSpec *pspec,
+                                         gpointer user_data);
+static void
+_fcitx_im_context_input_purpose_changed_cb(GObject *gobject, GParamSpec *pspec,
+                                           gpointer user_data);
+#endif
+
+
 static GdkEventKey *
 _create_gdk_event(FcitxIMContext *fcitxcontext,
                   guint keyval,
@@ -157,7 +160,7 @@ static gboolean
 _key_is_modifier(guint keyval);
 
 static void
-_request_surrounding_text (FcitxIMContext *context);
+_request_surrounding_text (FcitxIMContext **context);
 
 static gint
 _key_snooper_cb (GtkWidget   *widget,
@@ -336,8 +339,9 @@ fcitx_im_context_class_init(FcitxIMContextClass *klass)
 
 
 static void
-fcitx_im_context_class_fini (FcitxIMContextClass *klass)
+fcitx_im_context_class_fini(FcitxIMContextClass *klass)
 {
+    FCITX_UNUSED(klass);
     if (_key_snooper_id != 0) {
         gtk_key_snooper_remove (_key_snooper_id);
         _key_snooper_id = 0;
@@ -348,7 +352,7 @@ fcitx_im_context_class_fini (FcitxIMContextClass *klass)
 static void
 fcitx_im_context_init(FcitxIMContext *context)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_init");
+    FcitxLog(DEBUG, "fcitx_im_context_init");
     context->client = NULL;
     context->area.x = -1;
     context->area.y = -1;
@@ -356,6 +360,8 @@ fcitx_im_context_init(FcitxIMContext *context)
     context->area.height = 0;
     context->use_preedit = TRUE;
     context->cursor_pos = 0;
+    context->last_anchor_pos = -1;
+    context->last_cursor_pos = -1;
     context->preedit_string = NULL;
     context->attrlist = NULL;
     context->capacity = CAPACITY_SURROUNDING_TEXT;
@@ -391,6 +397,11 @@ fcitx_im_context_init(FcitxIMContext *context)
                      G_CALLBACK(_slave_delete_surrounding_cb),
                      context);
 
+#if GTK_CHECK_VERSION(3, 6, 0)
+    g_signal_connect(context, "notify::input-hints", G_CALLBACK(_fcitx_im_context_input_hints_changed_cb), NULL);
+    g_signal_connect(context, "notify::input-purpose", G_CALLBACK(_fcitx_im_context_input_purpose_changed_cb), NULL);
+#endif
+
     context->time = GDK_CURRENT_TIME;
 
     context->client = fcitx_client_new();
@@ -406,7 +417,7 @@ fcitx_im_context_init(FcitxIMContext *context)
 static void
 fcitx_im_context_finalize(GObject *obj)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_finalize");
+    FcitxLog(DEBUG, "fcitx_im_context_finalize");
     FcitxIMContext *context = FCITX_IM_CONTEXT(obj);
 
     fcitx_im_context_set_client_window(GTK_IM_CONTEXT(context), NULL);
@@ -428,10 +439,11 @@ fcitx_im_context_finalize(GObject *obj)
         context->slave = NULL;
     }
 
-    if (context->preedit_string) {
-        g_free(context->preedit_string);
-        context->preedit_string = NULL;
-    }
+    g_free(context->preedit_string);
+    context->preedit_string = NULL;
+
+    g_free(context->surrounding_text);
+    context->surrounding_text = NULL;
 
     if (context->attrlist) {
         pango_attr_list_unref(context->attrlist);
@@ -467,7 +479,7 @@ static void
 fcitx_im_context_set_client_window(GtkIMContext          *context,
                                    GdkWindow             *client_window)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_set_client_window");
+    FcitxLog(DEBUG, "fcitx_im_context_set_client_window");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
     set_ic_client_window(fcitxcontext, client_window);
 }
@@ -477,7 +489,7 @@ static gboolean
 fcitx_im_context_filter_keypress(GtkIMContext *context,
                                  GdkEventKey  *event)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_filter_keypress");
+    FcitxLog(DEBUG, "fcitx_im_context_filter_keypress");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     /* check this first, since we use key snooper, most key will be handled. */
@@ -502,7 +514,9 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
         return gtk_im_context_filter_keypress(fcitxcontext->slave, event);
 
     if (fcitx_client_is_valid(fcitxcontext->client) && fcitxcontext->has_focus) {
-        _request_surrounding_text (fcitxcontext);
+        _request_surrounding_text (&fcitxcontext);
+        if (G_UNLIKELY(!fcitxcontext))
+            return FALSE;
 
         fcitxcontext->time = event->time;
 
@@ -521,11 +535,6 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
                 return TRUE;
             }
         } else {
-            ProcessKeyStruct* pks = g_malloc0(sizeof(ProcessKeyStruct));
-            pks->context = fcitxcontext;
-            pks->event = (GdkEventKey *)  gdk_event_copy((GdkEvent *) event);
-
-            g_object_ref(fcitxcontext);
             fcitx_client_process_key_async(fcitxcontext->client,
                                            event->keyval,
                                            event->hardware_keycode,
@@ -535,7 +544,7 @@ fcitx_im_context_filter_keypress(GtkIMContext *context,
                                            -1,
                                            NULL,
                                            _fcitx_im_context_process_key_cb,
-                                           pks
+                                           gdk_event_copy((GdkEvent *) event)
                                           );
             event->state |= FcitxKeyState_HandledMask;
             return TRUE;
@@ -551,23 +560,21 @@ _fcitx_im_context_process_key_cb (GObject *source_object,
                                   GAsyncResult *res,
                                   gpointer user_data)
 {
-    ProcessKeyStruct* pks = user_data;
-    FcitxIMContext* context = pks->context;
-    GdkEventKey* event = pks->event;
+    GdkEventKey* event = user_data;
     int ret = fcitx_client_process_key_finish(FCITX_CLIENT(source_object), res);
     if (ret <= 0) {
         event->state |= FcitxKeyState_IgnoredMask;
         gdk_event_put((GdkEvent *)event);
     }
     gdk_event_free((GdkEvent *)event);
-    g_free(pks);
-    g_object_unref(context);
 }
 
 static void
-_fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array, int cursor_pos, void* user_data)
+_fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
+                                              int cursor_pos, void* user_data)
 {
-    FcitxLog(LOG_LEVEL, "_fcitx_im_context_commit_string_cb");
+    FCITX_UNUSED(im);
+    FcitxLog(DEBUG, "_fcitx_im_context_commit_string_cb");
     FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
 
     gboolean visible = false;
@@ -587,11 +594,11 @@ _fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
 
     GString* gstr = g_string_new(NULL);
 
-    int i = 0;
-    for (i = 0; i < array->len; i++) {
+    unsigned int i = 0;
+    for (i = 0;i < array->len;i++) {
         size_t bytelen = strlen(gstr->str);
-        FcitxPreeditItem* preedit = g_ptr_array_index(array, i);
-        const gchar* s = preedit->string;
+        FcitxPreeditItem *preedit = g_ptr_array_index(array, i);
+        const gchar *s = preedit->string;
         gint type = preedit->type;
 
         PangoAttribute *pango_attr = NULL;
@@ -603,7 +610,7 @@ _fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
         }
 
         if (type & MSG_HIGHLIGHT) {
-            gboolean hasColor;
+            gboolean hasColor = false;
             GdkColor fg;
             GdkColor bg;
             memset(&fg, 0, sizeof(GdkColor));
@@ -642,7 +649,7 @@ _fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
         gstr = g_string_append(gstr, s);
     }
 
-    gchar* str = g_string_free(gstr, FALSE);
+    gchar *str = g_string_free(gstr, FALSE);
 
     context->preedit_string = str;
     char* tempstr = g_strndup(str, cursor_pos);
@@ -683,7 +690,7 @@ _fcitx_im_context_update_formatted_preedit_cb(FcitxClient* im, GPtrArray* array,
 static void
 fcitx_im_context_focus_in(GtkIMContext *context)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_focus_in");
+    FcitxLog(DEBUG, "fcitx_im_context_focus_in");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     if (fcitxcontext->has_focus)
@@ -712,7 +719,9 @@ fcitx_im_context_focus_in(GtkIMContext *context)
                     g_object_ref(fcitxcontext),
                     (GDestroyNotify) g_object_unref);
 
-    _request_surrounding_text (fcitxcontext);
+    _request_surrounding_text (&fcitxcontext);
+    if (G_UNLIKELY(!fcitxcontext))
+        return;
 
     g_object_add_weak_pointer ((GObject *) context,
                                (gpointer *) &_focus_im_context);
@@ -724,7 +733,7 @@ fcitx_im_context_focus_in(GtkIMContext *context)
 static void
 fcitx_im_context_focus_out(GtkIMContext *context)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_focus_out");
+    FcitxLog(DEBUG, "fcitx_im_context_focus_out");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     if (!fcitxcontext->has_focus) {
@@ -760,7 +769,7 @@ static void
 fcitx_im_context_set_cursor_location(GtkIMContext *context,
                                      GdkRectangle *area)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_set_cursor_location %d %d %d %d", area->x, area->y, area->height, area->width);
+    FcitxLog(DEBUG, "fcitx_im_context_set_cursor_location %d %d %d %d", area->x, area->y, area->height, area->width);
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     if (fcitxcontext->area.x == area->x &&
@@ -824,7 +833,7 @@ static void
 fcitx_im_context_set_use_preedit(GtkIMContext *context,
                                  gboolean      use_preedit)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_set_use_preedit");
+    FcitxLog(DEBUG, "fcitx_im_context_set_use_preedit");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     fcitxcontext->use_preedit = use_preedit;
@@ -898,39 +907,53 @@ get_selection_anchor_point (FcitxIMContext *fcitxcontext,
 static void
 fcitx_im_context_set_surrounding (GtkIMContext *context,
                                   const gchar *text,
-                                  gint len,
+                                  gint l,
                                   gint cursor_index)
 {
     g_return_if_fail (context != NULL);
     g_return_if_fail (FCITX_IS_IM_CONTEXT (context));
     g_return_if_fail (text != NULL);
-    g_return_if_fail (strlen (text) >= len);
+
+    gint len = l;
+    if (len < 0) {
+        len = strlen(text);
+    }
+
     g_return_if_fail (0 <= cursor_index && cursor_index <= len);
 
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT (context);
 
-    if (fcitx_client_is_valid(fcitxcontext->client) && !(fcitxcontext->capacity & CAPACITY_PASSWORD)) {
-        guint cursor_pos;
+    if (fcitx_client_is_valid(fcitxcontext->client) &&
+        !(fcitxcontext->capacity & CAPACITY_PASSWORD)) {
+        gint cursor_pos;
         guint utf8_len;
         gchar *p;
 
-        p = g_strndup (text, len);
-        cursor_pos = g_utf8_strlen (p, cursor_index);
+        p = g_strndup(text, len);
+        cursor_pos = g_utf8_strlen(p, cursor_index);
         utf8_len = g_utf8_strlen(p, len);
 
-        guint anchor_pos = get_selection_anchor_point (fcitxcontext,
-                           cursor_pos,
-                           utf8_len);
-        fcitx_client_set_surrounding_text (fcitxcontext->client,
-                                           p,
-                                           cursor_pos,
-                                           anchor_pos);
-        g_free (p);
+        gint anchor_pos = get_selection_anchor_point(fcitxcontext,
+                                                     cursor_pos,
+                                                     utf8_len);
+        if (g_strcmp0(fcitxcontext->surrounding_text, p) == 0) {
+            g_free(p);
+            p = NULL;
+        } else {
+            g_free(fcitxcontext->surrounding_text);
+            fcitxcontext->surrounding_text = p;
+        }
+
+        if (p || fcitxcontext->last_cursor_pos != cursor_pos ||
+            fcitxcontext->last_anchor_pos != anchor_pos) {
+            fcitxcontext->last_cursor_pos = cursor_pos;
+            fcitxcontext->last_anchor_pos = anchor_pos;
+            fcitx_client_set_surrounding_text(fcitxcontext->client,
+                                              p, cursor_pos, anchor_pos);
+        }
     }
-    gtk_im_context_set_surrounding (fcitxcontext->slave,
-                                    text,
-                                    len,
-                                    cursor_index);
+    gtk_im_context_set_surrounding(fcitxcontext->slave,
+                                   text, l, cursor_index);
 }
 
 void
@@ -965,7 +988,7 @@ _fcitx_im_context_set_capacity(FcitxIMContext* fcitxcontext, gboolean force)
 static void
 fcitx_im_context_reset(GtkIMContext *context)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_reset");
+    FcitxLog(DEBUG, "fcitx_im_context_reset");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     if (fcitx_client_is_valid(fcitxcontext->client)) {
@@ -981,7 +1004,7 @@ fcitx_im_context_get_preedit_string(GtkIMContext   *context,
                                     PangoAttrList **attrs,
                                     gint           *cursor_pos)
 {
-    FcitxLog(LOG_LEVEL, "fcitx_im_context_get_preedit_string");
+    FcitxLog(DEBUG, "fcitx_im_context_get_preedit_string");
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
 
     if (fcitx_client_is_valid(fcitxcontext->client)) {
@@ -1021,12 +1044,14 @@ _slave_commit_cb(GtkIMContext *slave,
                  gchar *string,
                  FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     g_signal_emit(context, _signal_commit_id, 0, string);
 }
 static void
 _slave_preedit_changed_cb(GtkIMContext *slave,
                           FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     if (context->client) {
         return;
     }
@@ -1037,6 +1062,7 @@ static void
 _slave_preedit_start_cb(GtkIMContext *slave,
                         FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     if (context->client) {
         return;
     }
@@ -1048,6 +1074,7 @@ static void
 _slave_preedit_end_cb(GtkIMContext *slave,
                       FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     if (context->client) {
         return;
     }
@@ -1058,6 +1085,7 @@ static gboolean
 _slave_retrieve_surrounding_cb(GtkIMContext *slave,
                                FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     gboolean return_value;
 
     if (context->client) {
@@ -1074,22 +1102,27 @@ _slave_delete_surrounding_cb(GtkIMContext *slave,
                              guint nchars,
                              FcitxIMContext *context)
 {
+    FCITX_UNUSED(slave);
     gboolean return_value;
 
     if (context->client) {
         return FALSE;
     }
-    g_signal_emit(context, _signal_delete_surrounding_id, 0, offset_from_cursor, nchars, &return_value);
+    g_signal_emit(context, _signal_delete_surrounding_id, 0,
+                  offset_from_cursor, nchars, &return_value);
     return return_value;
 }
 
 void _fcitx_im_context_enable_im_cb(FcitxClient* im, void* user_data)
 {
+    FCITX_UNUSED(im);
+    FCITX_UNUSED(user_data);
 }
 
 void _fcitx_im_context_close_im_cb(FcitxClient* im, void* user_data)
 {
-    FcitxLog(LOG_LEVEL, "_fcitx_im_context_close_im_cb");
+    FCITX_UNUSED(im);
+    FcitxLog(DEBUG, "_fcitx_im_context_close_im_cb");
     FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
 
     if (context->preedit_string != NULL)
@@ -1102,14 +1135,16 @@ void _fcitx_im_context_close_im_cb(FcitxClient* im, void* user_data)
 
 void _fcitx_im_context_commit_string_cb(FcitxClient* im, char* str, void* user_data)
 {
-    FcitxLog(LOG_LEVEL, "_fcitx_im_context_commit_string_cb");
+    FCITX_UNUSED(im);
+    FcitxLog(DEBUG, "_fcitx_im_context_commit_string_cb");
     FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
     g_signal_emit(context, _signal_commit_id, 0, str);
 }
 
 void _fcitx_im_context_forward_key_cb(FcitxClient* im, guint keyval, guint state, gint type, void* user_data)
 {
-    FcitxLog(LOG_LEVEL, "_fcitx_im_context_forward_key_cb");
+    FCITX_UNUSED(im);
+    FcitxLog(DEBUG, "_fcitx_im_context_forward_key_cb");
     FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
     FcitxKeyEventType tp = (FcitxKeyEventType) type;
     GdkEventKey* event = _create_gdk_event(context, keyval, state, tp);
@@ -1119,28 +1154,27 @@ void _fcitx_im_context_forward_key_cb(FcitxClient* im, guint keyval, guint state
 }
 
 static void
-_fcitx_im_context_delete_surrounding_text_cb (FcitxClient* im,
-        gint offset_from_cursor,
-        guint nchars,
-        void* user_data)
+_fcitx_im_context_delete_surrounding_text_cb (FcitxClient *im,
+                                              gint offset_from_cursor,
+                                              guint nchars,
+                                              void *user_data)
 {
-    FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
+    FCITX_UNUSED(im);
+    FcitxIMContext *context =  FCITX_IM_CONTEXT(user_data);
     gboolean return_value;
-    g_signal_emit (context, _signal_delete_surrounding_id, 0, offset_from_cursor, nchars, &return_value);
+    g_signal_emit(context, _signal_delete_surrounding_id, 0,
+                  offset_from_cursor, nchars, &return_value);
 }
 
 /* Copy from gdk */
 static GdkEventKey *
-_create_gdk_event(FcitxIMContext *fcitxcontext,
-                  guint keyval,
-                  guint state,
-                  FcitxKeyEventType type
-                 )
+_create_gdk_event(FcitxIMContext *fcitxcontext, guint keyval, guint state,
+                  FcitxKeyEventType type)
 {
     gunichar c = 0;
     gchar buf[8];
 
-    GdkEventKey *event = (GdkEventKey *)gdk_event_new((type == FCITX_RELEASE_KEY) ? GDK_KEY_RELEASE : GDK_KEY_PRESS);
+    GdkEventKey *event = (GdkEventKey*)gdk_event_new((type == FCITX_RELEASE_KEY) ? GDK_KEY_RELEASE : GDK_KEY_PRESS);
 
     if (fcitxcontext && fcitxcontext->client_window)
         event->window = g_object_ref(fcitxcontext->client_window);
@@ -1320,6 +1354,7 @@ _key_is_modifier(guint keyval)
 
 void _fcitx_im_context_connect_cb(FcitxClient* im, void* user_data)
 {
+    FCITX_UNUSED(im);
     FcitxIMContext* context =  FCITX_IM_CONTEXT(user_data);
     _fcitx_im_context_set_capacity(context, TRUE);
     if (context->has_focus && _focus_im_context == (GtkIMContext*) context && fcitx_client_is_valid(context->client))
@@ -1334,21 +1369,31 @@ void _fcitx_im_context_connect_cb(FcitxClient* im, void* user_data)
 
 
 static void
-_request_surrounding_text (FcitxIMContext *context)
+_request_surrounding_text (FcitxIMContext **context)
 {
-    if (context && fcitx_client_is_valid(context->client)) {
+    if (*context && fcitx_client_is_valid((*context)->client)) {
         gboolean return_value;
-        FcitxLog(LOG_LEVEL, "requesting surrounding text");
-        g_signal_emit (context, _signal_retrieve_surrounding_id, 0,
+        FcitxLog(DEBUG, "requesting surrounding text");
+
+        /* according to RH#859879, something bad could happen here. */
+        g_object_add_weak_pointer ((GObject *) *context,
+                                   (gpointer *) context);
+        /* some unref can happen here */
+        g_signal_emit (*context, _signal_retrieve_surrounding_id, 0,
                        &return_value);
+        if (context)
+            g_object_remove_weak_pointer ((GObject *) *context,
+                                          (gpointer *) context);
+        else
+            return;
         if (return_value) {
-            context->capacity |= CAPACITY_SURROUNDING_TEXT;
-            _fcitx_im_context_set_capacity (context,
+            (*context)->capacity |= CAPACITY_SURROUNDING_TEXT;
+            _fcitx_im_context_set_capacity (*context,
                                             FALSE);
         }
         else {
-            context->capacity &= ~CAPACITY_SURROUNDING_TEXT;
-            _fcitx_im_context_set_capacity (context,
+            (*context)->capacity &= ~CAPACITY_SURROUNDING_TEXT;
+            _fcitx_im_context_set_capacity (*context,
                                             FALSE);
         }
     }
@@ -1356,13 +1401,13 @@ _request_surrounding_text (FcitxIMContext *context)
 
 
 static gint
-_key_snooper_cb (GtkWidget   *widget,
-                 GdkEventKey *event,
-                 gpointer     user_data)
+_key_snooper_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
+    FCITX_UNUSED(widget);
+    FCITX_UNUSED(user_data);
     gboolean retval = FALSE;
 
-    FcitxIMContext *fcitxcontext = (FcitxIMContext *) _focus_im_context;
+    FcitxIMContext *fcitxcontext = (FcitxIMContext*)_focus_im_context;
 
     if (G_UNLIKELY(!_use_key_snooper))
         return FALSE;
@@ -1381,15 +1426,9 @@ _key_snooper_cb (GtkWidget   *widget,
             break;
         }
 
-        /* according to RH#859879, something bad could happen here. */
-        g_object_add_weak_pointer ((GObject *) fcitxcontext,
-                                   (gpointer *) &fcitxcontext);
-        _request_surrounding_text (fcitxcontext);
+        _request_surrounding_text (&fcitxcontext);
         if (G_UNLIKELY(!fcitxcontext))
             return FALSE;
-        else
-            g_object_remove_weak_pointer ((GObject *) fcitxcontext,
-                                          (gpointer *) &fcitxcontext);
         fcitxcontext->time = event->time;
 
         if (_use_sync_mode) {
@@ -1405,11 +1444,6 @@ _key_snooper_cb (GtkWidget   *widget,
             else
                 retval = TRUE;
         } else {
-            ProcessKeyStruct* pks = g_malloc0(sizeof(ProcessKeyStruct));
-            pks->context = fcitxcontext;
-            pks->event = (GdkEventKey *)  gdk_event_copy((GdkEvent *) event);
-
-            g_object_ref(fcitxcontext);
             fcitx_client_process_key_async(fcitxcontext->client,
                                            event->keyval,
                                            event->hardware_keycode,
@@ -1419,7 +1453,7 @@ _key_snooper_cb (GtkWidget   *widget,
                                            -1,
                                            NULL,
                                            _fcitx_im_context_process_key_cb,
-                                           pks
+                                           gdk_event_copy((GdkEvent *) event)
                                           );
             retval = TRUE;
         }
@@ -1436,4 +1470,92 @@ _key_snooper_cb (GtkWidget   *widget,
     return retval;
 }
 
+#if GTK_CHECK_VERSION(3, 6, 0)
+
+void
+_fcitx_im_context_input_purpose_changed_cb(GObject* gobject, GParamSpec* pspec,
+                                           gpointer user_data)
+{
+    FCITX_UNUSED(pspec);
+    FCITX_UNUSED(user_data);
+    FcitxIMContext* fcitxcontext = FCITX_IM_CONTEXT(gobject);
+
+    GtkInputPurpose purpose;
+    g_object_get(gobject, "input-purpose", &purpose, NULL);
+
+    const FcitxCapacityFlags related_capacity =
+        CAPACITY_ALPHA |
+        CAPACITY_DIGIT |
+        CAPACITY_NUMBER |
+        CAPACITY_DIALABLE |
+        CAPACITY_URL |
+        CAPACITY_EMAIL |
+        CAPACITY_PASSWORD;
+
+    fcitxcontext->capacity &= ~related_capacity;
+
+#define CASE_PURPOSE(_PURPOSE, _CAPACITY) \
+    case _PURPOSE: \
+        fcitxcontext->capacity |= _CAPACITY; \
+        break;
+
+    switch(purpose) {
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_ALPHA, CAPACITY_ALPHA)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_DIGITS, CAPACITY_DIGIT);
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_NUMBER, CAPACITY_NUMBER)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_PHONE, CAPACITY_DIALABLE)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_URL, CAPACITY_URL)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_EMAIL, CAPACITY_EMAIL)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_NAME, CAPACITY_NAME)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_PASSWORD, CAPACITY_PASSWORD)
+        CASE_PURPOSE(GTK_INPUT_PURPOSE_PIN, CAPACITY_PASSWORD | CAPACITY_DIGIT)
+        case GTK_INPUT_PURPOSE_FREE_FORM:
+        default:
+            break;
+    }
+
+    _fcitx_im_context_set_capacity(fcitxcontext, FALSE);
+}
+
+void
+_fcitx_im_context_input_hints_changed_cb(GObject* gobject, GParamSpec *pspec,
+                                         gpointer user_data)
+{
+    FCITX_UNUSED(pspec);
+    FCITX_UNUSED(user_data);
+    FcitxIMContext* fcitxcontext = FCITX_IM_CONTEXT(gobject);
+
+    GtkInputHints hints;
+    g_object_get(gobject, "input-hints", &hints, NULL);
+
+    const FcitxCapacityFlags related_capacity =
+        CAPACITY_SPELLCHECK |
+        CAPACITY_NO_SPELLCHECK |
+        CAPACITY_WORD_COMPLETION |
+        CAPACITY_LOWERCASE |
+        CAPACITY_UPPERCASE |
+        CAPACITY_UPPERCASE_WORDS |
+        CAPACITY_UPPERCASE_SENTENCES |
+        CAPACITY_NO_ON_SCREEN_KEYBOARD;
+
+    fcitxcontext->capacity &= ~related_capacity;
+
+#define CHECK_HINTS(_HINTS, _CAPACITY) \
+    if (hints & _HINTS) \
+        fcitxcontext->capacity |= _CAPACITY;
+
+    CHECK_HINTS(GTK_INPUT_HINT_SPELLCHECK, CAPACITY_SPELLCHECK)
+    CHECK_HINTS(GTK_INPUT_HINT_NO_SPELLCHECK, CAPACITY_NO_SPELLCHECK);
+    CHECK_HINTS(GTK_INPUT_HINT_WORD_COMPLETION, CAPACITY_WORD_COMPLETION)
+    CHECK_HINTS(GTK_INPUT_HINT_LOWERCASE, CAPACITY_LOWERCASE)
+    CHECK_HINTS(GTK_INPUT_HINT_UPPERCASE_CHARS, CAPACITY_UPPERCASE)
+    CHECK_HINTS(GTK_INPUT_HINT_UPPERCASE_WORDS, CAPACITY_UPPERCASE_WORDS)
+    CHECK_HINTS(GTK_INPUT_HINT_UPPERCASE_SENTENCES, CAPACITY_UPPERCASE_SENTENCES)
+    CHECK_HINTS(GTK_INPUT_HINT_INHIBIT_OSK, CAPACITY_NO_ON_SCREEN_KEYBOARD)
+
+    _fcitx_im_context_set_capacity(fcitxcontext, FALSE);
+}
+
+#endif
+
 // kate: indent-mode cstyle; replace-tabs on;
