@@ -24,14 +24,25 @@
 #include "frontend/ipc/ipc.h"
 #include "fcitx/fcitx.h"
 #include "fcitxclient.h"
+#include "fcitxconnection.h"
 #include "marshall.h"
 
+#ifdef _DEBUG
 #define fcitx_gclient_debug(...) g_log ("fcitx-client",       \
                                       G_LOG_LEVEL_DEBUG,    \
                                       __VA_ARGS__)
 
+#else
+#define fcitx_gclient_debug(...)
+#endif
 typedef struct _ProcessKeyStruct ProcessKeyStruct;
 
+/**
+ * FcitxClient:
+ *
+ * A #FcitxClient allow to create a input context via DBus
+ */
+
 struct _ProcessKeyStruct {
     FcitxClient* self;
     GAsyncReadyCallback callback;
@@ -44,11 +55,8 @@ struct _FcitxClientPrivate {
     char servicename[64];
     char icname[64];
     int id;
-    guint watch_id;
-    GFileMonitor* monitor;
     GCancellable* cancellable;
-    GDBusConnection* connection;
-    gboolean connection_is_bus;
+    FcitxConnection* connection;
 };
 
 static const gchar introspection_xml[] =
@@ -95,6 +103,10 @@ static const gchar ic_introspection_xml[] =
     "      <arg name=\"cursor\" direction=\"in\" type=\"u\"/>\n"
     "      <arg name=\"anchor\" direction=\"in\" type=\"u\"/>\n"
     "    </method>\n"
+    "    <method name=\"SetSurroundingTextPosition\">\n"
+    "      <arg name=\"cursor\" direction=\"in\" type=\"u\"/>\n"
+    "      <arg name=\"anchor\" direction=\"in\" type=\"u\"/>\n"
+    "    </method>\n"
     "    <method name=\"DestroyIC\">\n"
     "    </method>\n"
     "    <method name=\"ProcessKeyEvent\">\n"
@@ -160,7 +172,8 @@ static guint signals[LAST_SIGNAL] = {0};
 
 static GDBusInterfaceInfo *_fcitx_client_get_interface_info(void);
 static GDBusInterfaceInfo *_fcitx_client_get_clientic_info(void);
-static void _fcitx_client_create_ic(FcitxClient* self, gboolean use_session_bus);
+static void _fcitx_client_create_ic(FcitxConnection* connection, gpointer user_data);
+static void _fcitx_client_disconnect(FcitxConnection* connection, gpointer user_data);
 static void _fcitx_client_create_ic_phase1_finished(GObject* source_object, GAsyncResult* res, gpointer user_data);
 static void _fcitx_client_create_ic_cb(GObject *source_object, GAsyncResult *res, gpointer user_data);
 static void _fcitx_client_create_ic_phase2_finished(GObject *source_object, GAsyncResult *res, gpointer user_data);
@@ -168,15 +181,7 @@ static void _fcitx_client_g_signal(GDBusProxy *proxy, gchar *sender_name, gchar
 static void fcitx_client_init(FcitxClient *self);
 static void fcitx_client_finalize(GObject *object);
 static void fcitx_client_dispose(GObject *object);
-static void _fcitx_client_appear (GDBusConnection *connection, const gchar *name, const gchar *name_owner, gpointer user_data);
-static void _fcitx_client_vanish (GDBusConnection *connection, const gchar *name, gpointer user_data);
-static void _fcitx_client_socket_file_changed_cb (GFileMonitor *monitor, GFile *file, GFile *other_file, GFileMonitorEvent event_type, gpointer            user_data);
-static gchar* _fcitx_get_address ();
-static void _fcitx_client_create_ic_phase0_bus_finished(GObject *source_object, GAsyncResult *res, gpointer user_data);
-static void _fcitx_client_create_ic_phase0_connection_finished(GObject *source_object, GAsyncResult *res, gpointer user_data);
 static void _fcitx_client_clean_up(FcitxClient* self, gboolean dont_emit_disconn);
-static void _fcitx_client_unwatch(FcitxClient* self);
-static void _fcitx_client_watch(FcitxClient* self);
 
 static void fcitx_client_class_init(FcitxClientClass *klass);
 
@@ -222,25 +227,29 @@ fcitx_client_dispose(GObject *object)
 {
     FcitxClient *self = FCITX_CLIENT(object);
 
-    if (self->priv->monitor) {
-        g_signal_handlers_disconnect_by_func(self->priv->monitor,
-                                             G_CALLBACK(_fcitx_client_socket_file_changed_cb),
-                                             self);
-        g_object_unref(self->priv->monitor);
-        self->priv->monitor= NULL;
-    }
-
     if (self->priv->icproxy) {
         g_dbus_proxy_call(self->priv->icproxy, "DestroyIC", NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
     }
-    _fcitx_client_unwatch(self);
 
+#ifndef g_signal_handlers_disconnect_by_data
+#define g_signal_handlers_disconnect_by_data(instance, data) \
+    g_signal_handlers_disconnect_matched ((instance), G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, (data))
+#endif
+    g_signal_handlers_disconnect_by_data(self->priv->connection,
+                                         self);
+    g_object_unref(self->priv->connection);
     _fcitx_client_clean_up(self, TRUE);
 
     if (G_OBJECT_CLASS(fcitx_client_parent_class)->dispose != NULL)
         G_OBJECT_CLASS(fcitx_client_parent_class)->dispose(object);
 }
 
+/**
+ * fcitx_client_enable_ic:
+ * @self: A #FcitxClient
+ *
+ * tell fcitx activate current ic
+ **/
 FCITX_EXPORT_API
 void fcitx_client_enable_ic(FcitxClient* self)
 {
@@ -249,6 +258,12 @@ void fcitx_client_enable_ic(FcitxClient* self)
     }
 }
 
+/**
+ * fcitx_client_close_ic:
+ * @self: A #FcitxClient
+ *
+ * tell fcitx inactivate current ic
+ **/
 FCITX_EXPORT_API
 void fcitx_client_close_ic(FcitxClient* self)
 {
@@ -257,6 +272,12 @@ void fcitx_client_close_ic(FcitxClient* self)
     }
 }
 
+/**
+ * fcitx_client_focus_in:
+ * @self: A #FcitxClient
+ *
+ * tell fcitx current client has focus
+ **/
 FCITX_EXPORT_API
 void fcitx_client_focus_in(FcitxClient* self)
 {
@@ -265,6 +286,12 @@ void fcitx_client_focus_in(FcitxClient* self)
     }
 }
 
+/**
+ * fcitx_client_focus_out:
+ * @self: A #FcitxClient
+ *
+ * tell fcitx current client has lost focus
+ **/
 FCITX_EXPORT_API
 void fcitx_client_focus_out(FcitxClient* self)
 {
@@ -273,6 +300,12 @@ void fcitx_client_focus_out(FcitxClient* self)
     }
 }
 
+/**
+ * fcitx_client_reset:
+ * @self: A #FcitxClient
+ *
+ * tell fcitx current client is reset from client side
+ **/
 FCITX_EXPORT_API
 void fcitx_client_reset(FcitxClient* self)
 {
@@ -281,6 +314,13 @@ void fcitx_client_reset(FcitxClient* self)
     }
 }
 
+/**
+ * fcitx_client_set_capacity:
+ * @self: A #FcitxClient
+ * @flags: capacity
+ *
+ * set client capacity of Fcitx
+ **/
 FCITX_EXPORT_API
 void fcitx_client_set_capacity(FcitxClient* self, guint flags)
 {
@@ -290,12 +330,34 @@ void fcitx_client_set_capacity(FcitxClient* self, guint flags)
     }
 }
 
+/**
+ * fcitx_client_set_cusor_rect:
+ * @self A #FcitxClient
+ * @x x of cursor
+ * @y y of cursor
+ * @w width of cursor
+ * @h height of cursor
+ *
+ * Deprecated:
+ *
+ * tell fcitx current client's cursor geometry info
+ **/
 FCITX_EXPORT_API
 void fcitx_client_set_cusor_rect(FcitxClient* self, int x, int y, int w, int h)
 {
     fcitx_client_set_cursor_rect(self, x, y, w, h);
 }
 
+/**
+ * fcitx_client_set_cursor_rect:
+ * @self: A #FcitxClient
+ * @x: x of cursor
+ * @y: y of cursor
+ * @w: width of cursor
+ * @h: height of cursor
+ *
+ * tell fcitx current client's cursor geometry info
+ **/
 FCITX_EXPORT_API
 void fcitx_client_set_cursor_rect(FcitxClient* self, int x, int y, int w, int h)
 {
@@ -304,14 +366,41 @@ void fcitx_client_set_cursor_rect(FcitxClient* self, int x, int y, int w, int h)
     }
 }
 
+/**
+ * fcitx_client_set_surrounding_text:
+ * @self: A #FcitxClient
+ * @text: (transfer none) (allow-none): surroundng text
+ * @cursor: cursor position coresponding to text
+ * @anchor: anchor position coresponding to text
+ **/
 FCITX_EXPORT_API
 void fcitx_client_set_surrounding_text(FcitxClient* self, gchar* text, guint cursor, guint anchor)
 {
     if (self->priv->icproxy) {
-        g_dbus_proxy_call(self->priv->icproxy, "SetSurroundingText", g_variant_new("(suu)", text, cursor, anchor), G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
+        if (text) {
+            g_dbus_proxy_call(self->priv->icproxy, "SetSurroundingText", g_variant_new("(suu)", text, cursor, anchor), G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
+        }
+        else {
+            g_dbus_proxy_call(self->priv->icproxy, "SetSurroundingTextPosition", g_variant_new("(uu)", cursor, anchor), G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
+        }
     }
 }
 
+/**
+ * fcitx_client_process_key:
+ * @self: A #FcitxClient
+ * @cb: callback
+ * @user_data: user data
+ * @keyval: key value
+ * @keycode: hardware key code
+ * @state: key state
+ * @type: event type
+ * @t: timestamp
+ *
+ * Deprecated:
+ *
+ * send a key event to fcitx asynchronizely, you need to use #g_dbus_proxy_call_finish with this function
+ **/
 FCITX_EXPORT_API
 void fcitx_client_process_key(FcitxClient* self, GAsyncReadyCallback cb, gpointer user_data, guint32 keyval, guint32 keycode, guint32 state, gint type, guint32 t)
 {
@@ -327,6 +416,15 @@ void fcitx_client_process_key(FcitxClient* self, GAsyncReadyCallback cb, gpointe
     }
 }
 
+/**
+ * fcitx_client_process_key_finish:
+ * @self: A #FcitxClient
+ * @res: result
+ *
+ * use this function with #fcitx_client_process_key_async
+ *
+ * Returns: process key result
+ **/
 FCITX_EXPORT_API
 gint fcitx_client_process_key_finish(FcitxClient* self, GAsyncResult* res)
 {
@@ -353,6 +451,7 @@ _fcitx_client_process_key_cb(GObject *source_object,
                              GAsyncResult *res,
                              gpointer user_data)
 {
+    FCITX_UNUSED(source_object);
     ProcessKeyStruct* pk = user_data;
     pk->callback(G_OBJECT(pk->self), res, pk->user_data);
     _process_key_data_free(pk);
@@ -360,13 +459,30 @@ _fcitx_client_process_key_cb(GObject *source_object,
 
 void _fcitx_client_process_key_cancelled(GCancellable* cancellable, gpointer user_data)
 {
+    FCITX_UNUSED(cancellable);
     ProcessKeyStruct* pk = user_data;
     _process_key_data_free(pk);
 }
 
+/**
+ * fcitx_client_process_key_async:
+ * @self: A #FcitxClient
+ * @keyval: key value
+ * @keycode: hardware key code
+ * @state: key state
+ * @type: event type
+ * @t: timestamp
+ * @timeout_msec: timeout in millisecond
+ * @cancellable: cancellable
+ * @callback: (scope async) (closure user_data): callback
+ * @user_data: (closure): user data
+ *
+ * use this function with #fcitx_client_process_key_finish
+ **/
 FCITX_EXPORT_API
 void fcitx_client_process_key_async(FcitxClient* self,
-                                    guint32 keyval, guint32 keycode, guint32 state, gint type, guint32 t,
+                                    guint32 keyval, guint32 keycode,
+                                    guint32 state, gint type, guint32 t,
                                     gint timeout_msec,
                                     GCancellable *cancellable,
                                     GAsyncReadyCallback callback,
@@ -389,6 +505,19 @@ void fcitx_client_process_key_async(FcitxClient* self,
     }
 }
 
+/**
+ * fcitx_client_process_key_sync:
+ * @self: A #FcitxClient
+ * @keyval: key value
+ * @keycode: hardware key code
+ * @state: key state
+ * @type: event type
+ * @t: timestamp
+ *
+ * send a key event to fcitx synchronizely
+ *
+ * Returns: the key is processed or not
+ */
 FCITX_EXPORT_API
 int fcitx_client_process_key_sync(FcitxClient* self, guint32 keyval, guint32 keycode, guint32 state, gint type, guint32 t)
 {
@@ -411,58 +540,6 @@ int fcitx_client_process_key_sync(FcitxClient* self, guint32 keyval, guint32 key
     return ret;
 }
 
-static gboolean
-_fcitx_client_new_service_appear(gpointer user_data)
-{
-    FcitxClient* self = (FcitxClient*) user_data;
-    if (!self->priv->connection || g_dbus_connection_is_closed(self->priv->connection))
-        _fcitx_client_create_ic(self, FALSE);
-    return FALSE;
-}
-
-static void
-_fcitx_client_appear (GDBusConnection *connection,
-                      const gchar     *name,
-                      const gchar     *name_owner,
-                      gpointer         user_data)
-{
-    FcitxClient* self = (FcitxClient*) user_data;
-    gboolean new_owner_good = name_owner && (name_owner[0] != '\0');
-    if (new_owner_good) {
-        g_timeout_add_full(G_PRIORITY_DEFAULT,
-                           100,
-                           _fcitx_client_new_service_appear,
-                           g_object_ref(self),
-                           g_object_unref);
-    }
-}
-
-static void
-_fcitx_client_vanish (GDBusConnection *connection,
-                      const gchar     *name,
-                      gpointer         user_data)
-{
-    FcitxClient* self = (FcitxClient*) user_data;
-    _fcitx_client_clean_up(self, FALSE);
-}
-
-static gchar*
-_fcitx_get_socket_path()
-{
-    char* machineId = dbus_get_local_machine_id();
-    gchar* path;
-    gchar* addressFile = g_strdup_printf("%s-%d", machineId, fcitx_utils_get_display_number());
-    dbus_free(machineId);
-
-    path = g_build_filename (g_get_user_config_dir (),
-            "fcitx",
-            "dbus",
-            addressFile,
-            NULL);
-    g_free(addressFile);
-    return path;
-}
-
 static void
 fcitx_client_init(FcitxClient *self)
 {
@@ -474,97 +551,25 @@ fcitx_client_init(FcitxClient *self)
     self->priv->cancellable = NULL;
     self->priv->improxy = NULL;
     self->priv->icproxy = NULL;
-    self->priv->watch_id = 0;
-    self->priv->connection_is_bus = FALSE;
+    self->priv->connection = fcitx_connection_new();
 
-    gchar* path = _fcitx_get_socket_path();
-    GFile* file = g_file_new_for_path(path);
-    self->priv->monitor = g_file_monitor_file(file, 0, NULL, NULL);
-
-    g_signal_connect (self->priv->monitor, "changed", (GCallback) _fcitx_client_socket_file_changed_cb, self);
-
-    g_object_unref(file);
-    g_free(path);
-
-    _fcitx_client_create_ic(self, FALSE);
+    g_signal_connect (self->priv->connection, "connected", (GCallback) _fcitx_client_create_ic, self);
+    g_signal_connect (self->priv->connection, "disconnected", (GCallback) _fcitx_client_disconnect, self);
 }
 
 static void
-_fcitx_client_create_ic(FcitxClient *self, gboolean use_session_bus)
+_fcitx_client_create_ic(FcitxConnection* connection, gpointer user_data)
 {
+    FCITX_UNUSED(connection);
     fcitx_gclient_debug("_fcitx_client_create_ic");
-    _fcitx_client_unwatch(self);
+    FcitxClient *self = user_data;
+
     _fcitx_client_clean_up(self, FALSE);
-    self->priv->cancellable = g_cancellable_new ();
 
     g_object_ref(self);
-    if (!use_session_bus) {
-        gchar* address = _fcitx_get_address();
-        /* a trick for prevent cancellable being called after finalize */
-        if (address) {
-            /* since we have a possible valid address here */
-            g_dbus_connection_new_for_address(address,
-                    G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT | G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
-                    NULL,
-                    self->priv->cancellable,
-                    _fcitx_client_create_ic_phase0_connection_finished,
-                    self);
-            g_free(address);
-            return;
-        }
-    }
-
-    _fcitx_client_watch(self);
-    g_bus_get(G_BUS_TYPE_SESSION,
-              self->priv->cancellable,
-              _fcitx_client_create_ic_phase0_bus_finished,
-              self
-            );
-};
-
-static void
-_fcitx_client_connection_closed(GDBusConnection *connection,
-                                gboolean         remote_peer_vanished,
-                                GError          *error,
-                                gpointer         user_data)
-{
-    fcitx_gclient_debug("_fcitx_client_connection_closed");
-    FcitxClient* self = (FcitxClient*) user_data;
-    _fcitx_client_clean_up(self, FALSE);
-
-    _fcitx_client_watch(self);
-}
-
-static void
-_fcitx_client_create_ic_phase0_bus_finished(GObject *source_object,
-                                            GAsyncResult *res,
-                                            gpointer user_data)
-{
-    fcitx_gclient_debug("_fcitx_client_create_ic_phase0_bus_finished");
-    g_return_if_fail (user_data != NULL);
-    g_return_if_fail (FCITX_IS_CLIENT(user_data));
-
-    FcitxClient* self = (FcitxClient*) user_data;
-    if (self->priv->cancellable) {
-        g_object_unref (self->priv->cancellable);
-        self->priv->cancellable = NULL;
-    }
-
-    GDBusConnection* connection = g_bus_get_finish(res, NULL);
-
-    if (!connection) {
-        /* unref for _fcitx_client_create_ic */
-        g_object_unref(self);
-        return;
-    }
-
-    _fcitx_client_clean_up(self, FALSE);
-    self->priv->connection = connection;
-    self->priv->connection_is_bus = TRUE;
-    g_signal_connect(connection, "closed", G_CALLBACK(_fcitx_client_connection_closed), self);
     self->priv->cancellable = g_cancellable_new ();
     g_dbus_proxy_new(
-        connection,
+        fcitx_connection_get_g_dbus_connection(self->priv->connection),
         G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
         _fcitx_client_get_interface_info(),
         self->priv->servicename,
@@ -577,87 +582,11 @@ _fcitx_client_create_ic_phase0_bus_finished(GObject *source_object,
 }
 
 static void
-_fcitx_client_watch(FcitxClient* self)
+_fcitx_client_disconnect(FcitxConnection* connection, gpointer user_data)
 {
-    if (self->priv->watch_id)
-        return;
-    fcitx_gclient_debug("_fcitx_client_watch");
-
-    self->priv->watch_id = g_bus_watch_name(
-                       G_BUS_TYPE_SESSION,
-                       self->priv->servicename,
-                       G_BUS_NAME_WATCHER_FLAGS_NONE,
-                       _fcitx_client_appear,
-                       _fcitx_client_vanish,
-                       self,
-                       NULL
-                   );
-}
-
-static void
-_fcitx_client_unwatch(FcitxClient* self)
-{
-    if (self->priv->watch_id)
-        g_bus_unwatch_name(self->priv->watch_id);
-    self->priv->watch_id = 0;
-}
-
-static void
-_fcitx_client_create_ic_phase0_connection_finished(GObject *source_object,
-                                                   GAsyncResult *res,
-                                                   gpointer user_data)
-
-{
-    fcitx_gclient_debug("_fcitx_client_create_ic_phase0_connection_finished");
-    g_return_if_fail (user_data != NULL);
-    g_return_if_fail (FCITX_IS_CLIENT(user_data));
-    FcitxClient* self = (FcitxClient*) user_data;
-    if (self->priv->cancellable) {
-        g_object_unref (self->priv->cancellable);
-        self->priv->cancellable = NULL;
-    }
-
-    GError* error = NULL;
-    GDBusConnection* connection = g_dbus_connection_new_for_address_finish(res, &error);
-
-    gboolean is_cancel = FALSE;
-    if (error) {
-        if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-            is_cancel = TRUE;
-        g_error_free(error);
-    }
-
-    /* hey! if we failed here. we'd try traditional dbus way */
-    if (!connection || g_dbus_connection_is_closed(connection)) {
-        if (connection)
-            g_object_unref(connection);
-
-        if (!is_cancel)
-            _fcitx_client_create_ic(self, TRUE);
-        /* unref for _fcitx_client_create_ic */
-        g_object_unref(self);
-        return;
-    }
-
-    g_dbus_connection_set_exit_on_close(connection, FALSE);
-
+    FCITX_UNUSED(connection);
+    FcitxClient *self = user_data;
     _fcitx_client_clean_up(self, FALSE);
-    self->priv->connection = connection;
-    self->priv->connection_is_bus = FALSE;
-    g_signal_connect(self->priv->connection, "closed", G_CALLBACK(_fcitx_client_connection_closed), self);
-
-    self->priv->cancellable = g_cancellable_new ();
-    g_dbus_proxy_new(
-        connection,
-        G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
-        _fcitx_client_get_interface_info(),
-        self->priv->servicename,
-        FCITX_IM_DBUS_PATH,
-        FCITX_IM_DBUS_INTERFACE,
-        self->priv->cancellable,
-        _fcitx_client_create_ic_phase1_finished,
-        self
-    );
 }
 
 static void
@@ -665,6 +594,7 @@ _fcitx_client_create_ic_phase1_finished(GObject *source_object,
                                         GAsyncResult *res,
                                         gpointer user_data)
 {
+    FCITX_UNUSED(source_object);
     fcitx_gclient_debug("_fcitx_client_create_ic_phase1_finished");
     g_return_if_fail (user_data != NULL);
     g_return_if_fail (FCITX_IS_CLIENT(user_data));
@@ -693,7 +623,7 @@ _fcitx_client_create_ic_phase1_finished(GObject *source_object,
     } while(0);
 
     if (!self->priv->improxy) {
-        /* unref for phase0 finish */
+        /* unref for create_ic */
         g_object_unref(self);
         return;
     }
@@ -742,7 +672,7 @@ _fcitx_client_create_ic_cb(GObject *source_object,
 
     self->priv->cancellable = g_cancellable_new ();
     g_dbus_proxy_new(
-        self->priv->connection,
+        fcitx_connection_get_g_dbus_connection(self->priv->connection),
         G_DBUS_PROXY_FLAGS_NONE,
         _fcitx_client_get_clientic_info(),
         self->priv->servicename,
@@ -760,6 +690,7 @@ _fcitx_client_create_ic_phase2_finished(GObject *source_object,
                                         GAsyncResult *res,
                                         gpointer user_data)
 {
+    FCITX_UNUSED(source_object);
     g_return_if_fail (user_data != NULL);
     g_return_if_fail (FCITX_IS_CLIENT(user_data));
     FcitxClient* self = (FcitxClient*) user_data;
@@ -809,6 +740,8 @@ _fcitx_client_g_signal(GDBusProxy *proxy,
                        GVariant   *parameters,
                        gpointer    user_data)
 {
+    FCITX_UNUSED(proxy);
+    FCITX_UNUSED(sender_name);
     if (strcmp(signal_name, "EnableIM") == 0) {
         g_signal_emit(user_data, signals[ENABLE_IM_SIGNAL], 0);
     }
@@ -874,9 +807,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
 
     /* install signals */
     /**
-     * FcitxClient::connected
-     *
-     * @client: A FcitxClient
+     * FcitxClient::connected:
+     * @self: A #FcitxClient
      *
      * Emit when connected to fcitx and created ic
      */
@@ -892,9 +824,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                      0);
 
     /**
-     * FcitxClient::disconnected
-     *
-     * @client: A FcitxClient
+     * FcitxClient::disconnected:
+     * @self: A #FcitxClient
      *
      * Emit when disconnected from fcitx
      */
@@ -910,9 +841,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                      0);
 
     /**
-     * FcitxClient::enable-im
-     *
-     * @client: A FcitxClient
+     * FcitxClient::enable-im:
+     * @self: A #FcitxClient
      *
      * Emit when input method is enabled
      */
@@ -927,9 +857,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                     G_TYPE_NONE,
                                     0);
     /**
-     * FcitxClient::enable-im
-     *
-     * @client: A FcitxClient
+     * FcitxClient::close-im:
+     * @self: A #FcitxClient
      *
      * Emit when input method is closed
      */
@@ -944,9 +873,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                    G_TYPE_NONE,
                                    0);
     /**
-     * FcitxClient::enable-im
-     *
-     * @client: A FcitxClient
+     * FcitxClient::forward-key:
+     * @self: A #FcitxClient
      * @keyval: key value
      * @state: key state
      * @type: event type
@@ -966,9 +894,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                       G_TYPE_UINT, G_TYPE_INT, G_TYPE_INT
                                   );
     /**
-     * FcitxClient::commit-string
-     *
-     * @client: A FcitxClient
+     * FcitxClient::commit-string:
+     * @self: A #FcitxClient
      * @string: string to be commited
      *
      * Emit when input method commit one string
@@ -987,9 +914,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
                                     );
 
     /**
-     * FcitxClient::delete-surrounding-text
-     *
-     * @client: A FcitxClient
+     * FcitxClient::delete-surrounding-text:
+     * @self: A #FcitxClient
      * @cursor: deletion start
      * @len: deletion length
      *
@@ -1009,9 +935,8 @@ fcitx_client_class_init(FcitxClientClass *klass)
             );
 
     /**
-     * FcitxClient::update-client-side-ui
-     *
-     * @client: A FcitxClient
+     * FcitxClient::update-client-side-ui:
+     * @self: A #FcitxClient
      * @auxup: (transfer none): Aux up string
      * @auxdown: (transfer none): Aux down string
      * @preedit: (transfer none): preedit string
@@ -1034,10 +959,9 @@ fcitx_client_class_init(FcitxClientClass *klass)
             G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
 
     /**
-     * FcitxClient::update-formatted-preedit
-     *
-     * @client: A FcitxClient
-     * @preedit: (transfer none) (element-type FcitxPreeditItem): An FcitxPreeditItem List
+     * FcitxClient::update-formatted-preedit:
+     * @self: A #FcitxClient
+     * @preedit: (transfer none) (element-type FcitxPreeditItem): An #FcitxPreeditItem List
      * @cursor: cursor postion by utf8 byte
      *
      * Emit when input method need to delete surrounding text
@@ -1056,20 +980,29 @@ fcitx_client_class_init(FcitxClientClass *klass)
             );
 }
 
+/**
+ * fcitx_client_new:
+ *
+ * New a #FcitxClient
+ *
+ * Returns: A newly allocated #FcitxClient
+ **/
 FCITX_EXPORT_API
 FcitxClient*
 fcitx_client_new()
 {
     FcitxClient* self = g_object_new(FCITX_TYPE_CLIENT, NULL);
-
-    if (self != NULL) {
-        return FCITX_CLIENT(self);
-    }
-    else
-        return NULL;
-    return self;
+    return FCITX_CLIENT(self);
 }
 
+/**
+ * fcitx_client_is_valid:
+ * @self: A #FcitxClient
+ *
+ * Check #FcitxClient is valid to communicate with Fcitx
+ *
+ * Returns: #FcitxClient is valid or not
+ **/
 FCITX_EXPORT_API
 gboolean
 fcitx_client_is_valid(FcitxClient* self)
@@ -1077,80 +1010,9 @@ fcitx_client_is_valid(FcitxClient* self)
     return self->priv->icproxy != NULL;
 }
 
-static void
-_fcitx_client_socket_file_changed_cb (GFileMonitor       *monitor,
-                                      GFile              *file,
-                                      GFile              *other_file,
-                                      GFileMonitorEvent   event_type,
-                                      gpointer            user_data)
-{
-    FcitxClient* client = user_data;
-    if (event_type != G_FILE_MONITOR_EVENT_CHANGED &&
-        event_type != G_FILE_MONITOR_EVENT_CREATED &&
-        event_type != G_FILE_MONITOR_EVENT_DELETED)
-        return;
-
-    _fcitx_client_create_ic(client, FALSE);
-}
-
-static gchar*
-_fcitx_get_address ()
-{
-    gchar* address = NULL;
-    address = g_strdup (g_getenv("FCITX_DBUS_ADDRESS"));
-    if (address)
-        return address;
-
-    gchar* path = _fcitx_get_socket_path();
-    FILE* fp = fopen(path, "r");
-    g_free(path);
-
-    if (!fp)
-        return NULL;
-
-    const int BUFSIZE = 1024;
-
-    char buffer[BUFSIZE];
-    size_t sz = fread(buffer, sizeof(char), BUFSIZE, fp);
-    fclose(fp);
-    if (sz == 0)
-        return NULL;
-    char* p = buffer;
-    while(*p)
-        p++;
-    size_t addrlen = p - buffer;
-    if (sz != addrlen + 2 * sizeof(pid_t) + 1)
-        return NULL;
-
-    /* skip '\0' */
-    p++;
-    pid_t *ppid = (pid_t*) p;
-    pid_t daemonpid = ppid[0];
-    pid_t fcitxpid = ppid[1];
-
-    if (!fcitx_utils_pid_exists(daemonpid)
-        || !fcitx_utils_pid_exists(fcitxpid))
-        return NULL;
-
-    address = g_strdup(buffer);
-
-    return address;
-}
-
 static void
 _fcitx_client_clean_up(FcitxClient* self, gboolean dont_emit_disconn)
 {
-    if (self->priv->connection) {
-        g_signal_handlers_disconnect_by_func(self->priv->connection,
-                                             G_CALLBACK(_fcitx_client_connection_closed),
-                                             self);
-        if (!self->priv->connection_is_bus) {
-            g_dbus_connection_close_sync(self->priv->connection, NULL, NULL);
-        }
-        g_object_unref(self->priv->connection);
-        self->priv->connection = NULL;
-    }
-
     if (self->priv->cancellable) {
         g_cancellable_cancel (self->priv->cancellable);
         g_object_unref (self->priv->cancellable);
