@@ -37,6 +37,7 @@
 #include "instance-internal.h"
 #include "addon-internal.h"
 #include "ui-internal.h"
+#include "candidate-internal.h"
 
 /**
  * @file ui.c
@@ -185,7 +186,7 @@ boolean FcitxUILoadInternal(FcitxInstance* instance, FcitxAddon* addon)
 
         fclose(fp);
 
-        handle = dlopen(modulePath, RTLD_NOW | RTLD_GLOBAL);
+        handle = dlopen(modulePath, RTLD_NOW | (addon->loadLocal ? RTLD_LOCAL : RTLD_GLOBAL));
 
         if (!handle) {
             FcitxLog(ERROR, _("UI: open %s fail %s") , modulePath , dlerror());
@@ -350,13 +351,13 @@ FCITX_EXPORT_API
 void FcitxUICloseInputWindow(FcitxInstance* instance)
 {
     FcitxInstanceCleanInputWindow(instance);
-    instance->uiflag |= UI_UPDATE;
+    instance->eventflag |= FEF_UI_UPDATE;
 }
 
 FCITX_EXPORT_API
 void FcitxUIUpdateInputWindow(FcitxInstance *instance)
 {
-    instance->uiflag |= UI_UPDATE;
+    instance->eventflag |= FEF_UI_UPDATE;
 }
 
 void FcitxUIShowInputWindow(FcitxInstance* instance)
@@ -368,7 +369,7 @@ void FcitxUIShowInputWindow(FcitxInstance* instance)
 FCITX_EXPORT_API
 void FcitxUIMoveInputWindow(FcitxInstance* instance)
 {
-    instance->uiflag |= UI_MOVE;
+    instance->eventflag |= FEF_UI_MOVE;
 }
 
 FCITX_EXPORT_API
@@ -590,6 +591,34 @@ void FcitxUIRegisterMenu(FcitxInstance* instance, FcitxUIMenu* menu)
         instance->uifallback->ui->RegisterMenu(instance->uifallback->addonInstance, menu);
 }
 
+FCITX_EXPORT_API
+void FcitxUIUnRegisterMenu(FcitxInstance* instance, FcitxUIMenu* menu)
+{
+    UT_array* uimenus = &instance->uimenus;
+
+    if (!menu)
+        return;
+
+    utarray_foreach(menup, uimenus, FcitxUIMenu*) {
+        if (*menup == menu) {
+            utarray_remove_quick(uimenus, utarray_eltidx(uimenus, menup));
+
+            if (UI_FUNC_IS_VALID(UnRegisterMenu))
+                instance->ui->ui->UnRegisterMenu(instance->ui->addonInstance, menu);
+            if (UI_FUNC_IS_VALID_FALLBACK(UnRegisterMenu))
+                instance->uifallback->ui->UnRegisterMenu(instance->uifallback->addonInstance, menu);
+            break;
+        }
+    }
+
+}
+
+FCITX_EXPORT_API
+void FcitxMenuFinalize(FcitxUIMenu* menu)
+{
+    utarray_done(&menu->shell);
+}
+
 FCITX_EXPORT_API
 void FcitxMenuAddMenuItemWithData(FcitxUIMenu* menu, const char* string, FcitxMenuItemType type, FcitxUIMenu* subMenu, void* arg)
 {
@@ -764,6 +793,15 @@ void FcitxUIGetMainWindowSize(FcitxInstance* instance, int* x, int* y, int* w, i
         instance->ui->ui->MainWindowSizeHint(instance->ui->addonInstance, x, y, w, h);
 }
 
+static inline boolean FcitxUIUseDefaultHighlight(FcitxInstance* instance, FcitxCandidateWordList* candList)
+{
+    if (candList->overrideHighlight) {
+        return candList->overrideHighlightValue;
+    } else {
+        return !FcitxInstanceGetContextBoolean(instance, CONTEXT_DISABLE_AUTO_FIRST_CANDIDATE_HIGHTLIGHT);
+    }
+}
+
 FCITX_EXPORT_API
 int FcitxUINewMessageToOldStyleMessage(FcitxInstance* instance, FcitxMessages* msgUp, FcitxMessages* msgDown)
 {
@@ -811,9 +849,10 @@ int FcitxUINewMessageToOldStyleMessage(FcitxInstance* instance, FcitxMessages* m
         if (i == 0
             && FcitxCandidateWordGetCurrentPage(input->candList) == 0
             && type == MSG_OTHER
-            && !FcitxInstanceGetContextBoolean(instance, CONTEXT_DISABLE_AUTO_FIRST_CANDIDATE_HIGHTLIGHT)
-        )
+            && FcitxUIUseDefaultHighlight(instance, input->candList)
+        ) {
             type = MSG_FIRSTCAND;
+        }
 
         FcitxMessagesAddMessageStringsAtLast(msgDown, type, candWord->strWord);
 
@@ -1054,11 +1093,11 @@ FcitxUIMenu* FcitxUIGetMenuByStatusName(FcitxInstance* instance, const char* nam
             return NULL;
     }
 
-    UT_array* uimenus = FcitxInstanceGetUIMenus(instance);
+    UT_array* uimenus = &instance->uimenus;
     FcitxUIMenu** menupp, *menup = NULL;
     for (menupp = (FcitxUIMenu **) utarray_front(uimenus);
-            menupp != NULL;
-            menupp = (FcitxUIMenu **) utarray_next(uimenus, menupp)
+         menupp != NULL;
+         menupp = (FcitxUIMenu **) utarray_next(uimenus, menupp)
         ) {
         if ((*menupp)->candStatusBind && strcmp((*menupp)->candStatusBind, name) == 0) {
             menup = *menupp;
