@@ -38,6 +38,7 @@
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/utils.h"
 #include "fcitx-utils/bitset.h"
+#include "module/freedesktop-notify/fcitx-freedesktop-notify.h"
 
 /**
  * @file punc.c
@@ -289,9 +290,9 @@ boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN
     sym = FcitxHotkeyPadToMain(sym);
     if (profile->bUseWidePunc) {
         if (puncState->bLastIsNumber && config->bEngPuncAfterNumber
-                && (FcitxHotkeyIsHotKey(origsym, state, FCITX_PERIOD)
-                    || FcitxHotkeyIsHotKey(origsym, state, FCITX_SEMICOLON)
-                    || FcitxHotkeyIsHotKey(origsym, state, FCITX_COMMA))) {
+            && (FcitxHotkeyIsHotKey(origsym, state, FCITX_PERIOD)
+                || FcitxHotkeyIsHotKey(origsym, state, FCITX_SEMICOLON)
+                || FcitxHotkeyIsHotKey(origsym, state, FCITX_COMMA))) {
             puncState->cLastIsAutoConvert = origsym;
             puncState->bLastIsNumber = false;
             *retVal = IRV_DONOT_PROCESS;
@@ -337,7 +338,7 @@ boolean ProcessPunc(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN
 
     if (profile->bUseWidePunc) {
         if (FcitxHotkeyIsHotKey(sym, state, FCITX_BACKSPACE)
-                && puncState->cLastIsAutoConvert) {
+            && puncState->cLastIsAutoConvert) {
             char *pPunc;
 
             FcitxInstanceForwardKey(puncState->owner, FcitxInstanceGetCurrentIC(instance), FCITX_PRESS_KEY, sym, state);
@@ -559,15 +560,23 @@ void TogglePuncState(void* arg)
 
 INPUT_RETURN_VALUE TogglePuncStateWithHotkey(void* arg)
 {
-    FcitxPuncState* puncState = (FcitxPuncState*)arg;
+    FcitxPuncState *puncState = (FcitxPuncState*)arg;
+    FcitxInstance *instance = puncState->owner;
+    FcitxProfile *profile = FcitxInstanceGetProfile(instance);
 
-    FcitxUIStatus *status = FcitxUIGetStatusByName(puncState->owner, "punc");
+    FcitxUIStatus *status = FcitxUIGetStatusByName(instance, "punc");
     if (status->visible){
-        FcitxUIUpdateStatus(puncState->owner, "punc");
+        FcitxUIUpdateStatus(instance, "punc");
+        FcitxFreeDesktopNotifyShowAddonTip(
+            instance, "fcitx-punc-toggle",
+            profile->bUseWidePunc ? "fcitx-punc-active" : "fcitx-punc-inactive",
+            _("Punctuation Support"),
+            profile->bUseWidePunc ? _("Full width punctuations are used.") :
+                                    _("Latin punctuations are used."));
         return IRV_DO_NOTHING;
-    }
-    else
+    } else {
         return IRV_TO_PROCESS;
+    }
 }
 
 boolean GetPuncState(void* arg)
@@ -590,10 +599,10 @@ void ReloadPunc(void* arg)
 boolean IsHotKeyPunc(FcitxKeySym sym, unsigned int state)
 {
     if (FcitxHotkeyIsHotKeySimple(sym, state)
-            && !FcitxHotkeyIsHotKeyDigit(sym, state)
-            && !FcitxHotkeyIsHotKeyLAZ(sym, state)
-            && !FcitxHotkeyIsHotKeyUAZ(sym, state)
-            && !FcitxHotkeyIsHotKey(sym, state, FCITX_SPACE))
+        && !FcitxHotkeyIsHotKeyDigit(sym, state)
+        && !FcitxHotkeyIsHotKeyLAZ(sym, state)
+        && !FcitxHotkeyIsHotKeyUAZ(sym, state)
+        && !FcitxHotkeyIsHotKey(sym, state, FCITX_SPACE))
         return true;
 
     return false;
