@@ -94,12 +94,12 @@
 # 32bit: offset to unihan_strings for Korean
 # 32bit: offset to unihan_strings for JapaneseKun
 # 32bit: offset to unihan_strings for JapaneseOn
- 
+
 from struct import *
 import sys
 import re
-import StringIO
- 
+import io
+
 # based on http://www.unicode.org/charts/
 sectiondata = '''
 SECTION European Alphabets
@@ -286,9 +286,75 @@ Low Surrogates
 Private Use Area
 Specials
 Variation Selectors
+
+SECTION SMP
+Linear B Syllabary
+Linear B Ideograms
+Aegean Numbers
+Ancient Greek Numbers
+Ancient Symbols
+Phaistos Disc
+Lycian
+Carian
+Old Italic
+Gothic
+Ugaritic
+Old Persian
+Deseret
+Shavian
+Osmanya
+Cypriot Syllabary
+Imperial Aramaic
+Phoenician
+Lydian
+Meroitic Hieroglyphs
+Meroitic Cursive
+Kharoshthi
+Old South Arabian
+Avestan
+Inscriptional Parthian
+Inscriptional Pahlavi
+Old Turkic
+Rumi Numeral Symbols
+Brahmi
+Kaithi
+Sora Sompeng
+Chakma
+Sharada
+Takri
+Cuneiform
+Cuneiform Numbers and Punctuation
+Egyptian Hieroglyphs
+Bamum Supplement
+Miao
+Kana Supplement
+Byzantine Musical Symbols
+Musical Symbols
+Ancient Greek Musical Notation
+Tai Xuan Jing Symbols
+Counting Rod Numerals
+Mathematical Alphanumeric Symbols
+Arabic Mathematical Alphabetic Symbols
+Mahjong Tiles
+Domino Tiles
+Playing Cards
+Enclosed Alphanumeric Supplement
+Enclosed Ideographic Supplement
+Miscellaneous Symbols And Pictographs
+Emoticons
+Transport And Map Symbols
+Alchemical Symbols
+CJK Unified Ideographs Extension B
+CJK Unified Ideographs Extension C
+CJK Unified Ideographs Extension D
+CJK Compatibility Ideographs Supplement
+Tags
+Variation Selectors Supplement
+Supplementary Private Use Area-A
+Supplementary Private Use Area-B
 '''
 # TODO: rename "Other Scripts" to "American Scripts"
- 
+
 categoryMap = { # same values as QChar::Category
     "Mn": 1,
     "Mc": 2,
@@ -321,38 +387,38 @@ categoryMap = { # same values as QChar::Category
     "Sk":  29,
     "So":  30
 }
- 
- 
+
+
 class Names:
     def __init__(self):
         self.names = []
         self.controlpos = -1
     def addName(self, uni, name, category):
         self.names.append([uni, name, category])
- 
+
     def calculateStringSize(self):
         size = 0
         hadcontrol = False
         for entry in self.names:
             if entry[1] == "<control>":
                 if not hadcontrol:
-                    size += len(entry[1]) + 2
+                    size += len(entry[1].encode('utf-8')) + 2
                     hadcontrol = True
             else:
-                size += len(entry[1]) + 2
+                size += len(entry[1].encode('utf-8')) + 2
         return size
- 
+
     def calculateOffsetSize(self):
-        return len(self.names)*6
- 
+        return len(self.names)*8
+
     def writeStrings(self, out, pos):
         hadcontrol = False
         for entry in self.names:
             if entry[1] == "<control>":
                 if not hadcontrol:
                     out.write(pack("=b", entry[2]))
-                    out.write(entry[1] + "\0")
-                    size = len(entry[1]) + 2
+                    out.write(entry[1].encode('utf-8') + b"\0")
+                    size = len(entry[1].encode('utf-8')) + 2
                     entry[1] = pos
                     self.controlpos = pos
                     pos += size
@@ -361,57 +427,57 @@ class Names:
                     entry[1] = self.controlpos
             else:
                 out.write(pack("=b", entry[2]))
-                out.write(entry[1] + "\0")
-                size = len(entry[1]) + 2
+                out.write(entry[1].encode('utf-8') + b"\0")
+                size = len(entry[1].encode('utf-8')) + 2
                 entry[1] = pos
                 pos += size
         return pos
- 
+
     def writeOffsets(self, out, pos):
         for entry in self.names:
-            out.write(pack("=HI", int(entry[0], 16), entry[1]))
-            pos += 6
+            out.write(pack("=II", int(entry[0], 16), entry[1]))
+            pos += 8
         return pos
- 
+
 class Details:
     def __init__(self):
         self.details = {}
     def addEntry(self, char, category, text):
-        if not self.details.has_key(char):
+        if char not in self.details:
             self.details[char] = {}
-        if not self.details[char].has_key(category):
+        if category not in self.details[char]:
             self.details[char][category] = []
         self.details[char][category].append(text)
- 
+
     def calculateStringSize(self):
         size = 0
         for char in self.details.values():
             for cat in char.values():
                 for s in cat:
                     if type(s) is str:
-                        size += len(s) + 1
+                        size += len(s.encode('utf-8')) + 1
                     else:
-                        size += 2
+                        size += 4
         return size
- 
+
     def calculateOffsetSize(self):
-        return len(self.details)*27
- 
+        return len(self.details)*29
+
     def writeStrings(self, out, pos):
         for char in self.details.values():
             for cat in char.values():
                 for i in range(0, len(cat)):
                     s = cat[i]
                     if type(s) is str:
-                        out.write(s + "\0")
-                        size = len(s) + 1
+                        out.write(s.encode('utf-8') + b"\0")
+                        size = len(s.encode('utf-8')) + 1
                     else:
-                        out.write(pack("=H", s))
-                        size = 2
+                        out.write(pack("=I", s))
+                        size = 4
                     cat[i] = pos
                     pos += size
         return pos
- 
+
     def writeOffsets(self, out, pos):
         for char in self.details.keys():
             alias = 0
@@ -424,94 +490,94 @@ class Details:
             equiv_count = 0
             seeAlso = 0
             seeAlso_count = 0
-            if self.details[char].has_key("alias"):
+            if "alias" in self.details[char]:
                 alias = self.details[char]["alias"][0]
                 alias_count = len(self.details[char]["alias"])
- 
-            if self.details[char].has_key("note"):
+
+            if "note" in self.details[char]:
                 note = self.details[char]["note"][0]
                 note_count = len(self.details[char]["note"])
- 
-            if self.details[char].has_key("approxEquiv"):
+
+            if "approxEquiv" in self.details[char]:
                 approxEquiv = self.details[char]["approxEquiv"][0]
                 approxEquiv_count = len(self.details[char]["approxEquiv"])
- 
-            if self.details[char].has_key("equiv"):
+
+            if "equiv" in self.details[char]:
                 equiv = self.details[char]["equiv"][0]
                 equiv_count = len(self.details[char]["equiv"])
- 
-            if self.details[char].has_key("seeAlso"):
+
+            if "seeAlso" in self.details[char]:
                 seeAlso = self.details[char]["seeAlso"][0]
                 seeAlso_count = len(self.details[char]["seeAlso"])
- 
-            out.write(pack("=HIbIbIbIbIb", char, alias, alias_count, note, note_count, approxEquiv, approxEquiv_count, equiv, equiv_count, seeAlso, seeAlso_count))
-            pos += 27
- 
+
+            out.write(pack("=IIbIbIbIbIb", char, alias, alias_count, note, note_count, approxEquiv, approxEquiv_count, equiv, equiv_count, seeAlso, seeAlso_count))
+            pos += 29
+
         return pos
- 
+
 class SectionsBlocks:
     def __init__(self):
         self.sections = []
         self.blocks = []
         self.blockList = []
         self.sectionList = []
- 
+
     def addBlock(self, begin, end, name):
         self.blocks.append([begin, end, name])
         self.blockList.append(name)
- 
+
     def addSection(self, section, block):
         self.sections.append([section, block])
         if not section in self.sectionList:
             self.sectionList.append(section)
- 
+
     def calculateBlockStringSize(self):
         size = 0
         for block in self.blocks:
-            size += len(block[2]) + 1
+            size += len(block[2].encode('utf-8')) + 1
         return size
- 
+
     def calculateBlockOffsetSize(self):
-        return len(self.blocks) * 4
- 
+        return len(self.blocks) * 8
+
     def calculateSectionStringSize(self):
         size = 0
         lastsection = ""
         for section in self.sections:
             if section[0] != lastsection:
-                size += len(section[0]) + 1
+                size += len(section[0].encode('utf-8')) + 1
                 lastsection = section[0]
         return size
- 
+
     def calculateSectionOffsetSize(self):
-        return len(self.sections) * 4
- 
+        return len(self.sections) * 8
+
     def writeBlockStrings(self, out, pos):
         index = 0
         for block in self.blocks:
-            out.write(block[2] + "\0")
-            size = len(block[2]) + 1
+            out.write(block[2].encode('utf-8') + b"\0")
+            size = len(block[2].encode('utf-8')) + 1
             found = False
             for section in self.sections:
-                print section
+                print(section)
                 if section[1] == block[2]:
-                    print "found", section
+                    print("found", section)
                     section[1] = int(index)
                     found = True
             if not found:
-                print "Error: Did not find any category for block \""+block[2]+"\""
+                print("Error: Did not find any category for block \""+block[2]+"\"")
                 sys.exit(1)
             block[2] = index
             pos += size
             index += 1
         return pos
- 
+
     def writeBlockOffsets(self, out, pos):
         for block in self.blocks:
-            out.write(pack("=HH", int(block[0], 16), int(block[1], 16)))
-            pos += 4
+            out.write(pack("=II", int(block[0], 16), int(block[1], 16)))
+            pos += 8
         return pos
- 
+
     def writeSectionStrings(self, out, pos):
         lastsection = ""
         lastpos = 0
@@ -520,37 +586,37 @@ class SectionsBlocks:
             if section[0] != lastsection:
                 index += 1
                 lastsection = section[0]
-                out.write(section[0] + "\0")
-                size = len(section[0]) + 1
+                out.write(section[0].encode('utf-8') + b"\0")
+                size = len(section[0].encode('utf-8')) + 1
                 section[0] = index
                 lastpos = pos
                 pos += size
             else:
                 section[0] = index
         return pos
- 
+
     def writeSectionOffsets(self, out, pos):
-        print self.sections
+        print(self.sections)
         for section in self.sections:
-            out.write(pack("=HH", section[0], section[1]))
-            pos += 4
+            out.write(pack("=II", section[0], section[1]))
+            pos += 8
         return pos
- 
+
     def getBlockList(self):
         return self.blockList
-    
+
     def getSectionList(self):
         return self.sectionList
- 
+
 class Unihan:
     def __init__(self):
         self.unihan = {}
- 
+
     def addUnihan(self, uni, category, value):
         uni = int(uni, 16)
         if category != "kDefinition" and category != "kCantonese" and category != "kMandarin" and category != "kTang" and category != "kKorean" and category != "kJapaneseKun" and category != "kJapaneseOn":
             return
-        if not self.unihan.has_key(uni):
+        if uni not in self.unihan:
             self.unihan[uni] = [None, None, None, None, None, None, None]
         if category == "kDefinition":
             self.unihan[uni][0] = value
@@ -566,39 +632,39 @@ class Unihan:
             self.unihan[uni][5] = value
         elif category == "kJapaneseOn":
             self.unihan[uni][6] = value
- 
+
     def calculateStringSize(self):
         size = 0
         for char in self.unihan.keys():
             for entry in self.unihan[char]:
                 if entry != None:
-                    size += len(entry) + 1
+                    size += len(entry.encode('utf-8')) + 1
         return size
- 
+
     def calculateOffsetSize(self):
-        return len(self.unihan) * 30
- 
+        return len(self.unihan) * 32
+
     def writeStrings(self, out, pos):
         for char in self.unihan.keys():
             for i in range(0, 7):
                 if self.unihan[char][i] != None:
-                    out.write(self.unihan[char][i] + "\0")
-                    size = len(self.unihan[char][i]) + 1
+                    out.write(self.unihan[char][i].encode('utf-8') + b"\0")
+                    size = len(self.unihan[char][i].encode('utf-8')) + 1
                     self.unihan[char][i] = pos
                     pos += size
         return pos
- 
+
     def writeOffsets(self, out, pos):
         for char in self.unihan.keys():
-            out.write(pack("=H", char))
+            out.write(pack("=I", char))
             for i in range(0, 7):
                 if self.unihan[char][i] != None:
                     out.write(pack("=I", self.unihan[char][i]))
                 else:
                     out.write(pack("=I", 0))
-            pos += 30
+            pos += 32
         return pos
- 
+
 class Parser:
     def parseUnicodeData(self, inUnicodeData, names):
         regexp = re.compile(r'^([^;]+);([^;]+);([^;]+)')
@@ -610,23 +676,23 @@ class Parser:
             uni = m.group(1)
             name = m.group(2)
             category = m.group(3)
-            if len(uni) > 4:
+            if len(uni) > 8:
                 continue
             names.addName(uni, name, categoryMap[category])
- 
+
     def parseDetails(self, inNamesList, details):
         invalidRegexp = re.compile(r'^@')
         unicodeRegexp = re.compile(r'^([0-9A-F]+)')
- 
+
         aliasRegexp = re.compile(r'^\s+=\s+(.+)$') #equal
         seeAlsoRegexp = re.compile(r'^\s+x\s+.*([0-9A-F]{4,6})\)$') #ex
         noteRegexp = re.compile(r'^\s+\*\s+(.+)$') #star
         approxEquivalentRegexp = re.compile(r'^\s+#\s+(.+)$') #pound
         equivalentRegexp = re.compile(r'^\s+:\s+(.+)$') #colon
- 
+
         drop = 0
         currChar = 0
- 
+
         for line in inNamesList:
             line = line[:-1]
             m1 = unicodeRegexp.match(line)
@@ -639,7 +705,7 @@ class Parser:
                 continue
             elif m1:
                 currChar = int(m1.group(1), 16)
-                if len(m1.group(1)) > 4: #limit to 16bit
+                if len(m1.group(1)) > 8: #limit to 32bit
                     drop = 1
                     continue
             elif drop == 1:
@@ -666,7 +732,7 @@ class Parser:
             m = regexp.match(line)
             if not m:
                 continue
-            if len(m.group(1)) > 4:
+            if len(m.group(1)) > 8:
                 continue
             sectionsBlocks.addBlock(m.group(1), m.group(2), m.group(3))
     def parseSections(self, inSections, sectionsBlocks):
@@ -674,21 +740,22 @@ class Parser:
         for line in inSections:
             line = line[:-1]
             if len(line) == 0:
-                continue;
+                continue
             temp = line.split(" ")
             if temp[0] == "SECTION":
                 currSection = line[8:]
             elif currSection != "" and line != "":
                 sectionsBlocks.addSection(currSection, line)
             else:
-                print "error in data file"
+                print("error in data file")
                 sys.exit(1)
     def parseUnihan(self, inUnihan, unihan):
         regexp = re.compile(r'^U\+([0-9A-F]+)\s+([^\s]+)\s+(.+)$')
         count = 0
         for line in inUnihan:
             if count % 100000 == 0:
-                print "\b.",; sys.stdout.flush()
+                print("\b.", end=' ')
+                sys.stdout.flush()
             count += 1
             line = line[:-1]
             m = regexp.match(line)
@@ -696,141 +763,145 @@ class Parser:
                 continue
             if len(m.group(1)) <= 4:
                 unihan.addUnihan(m.group(1), m.group(2), m.group(3))
- 
+
 def writeTranslationDummy(out, data):
-    out.write("""/* This file is part of the KDE libraries
- 
+    out.write(b"""/* This file is part of the KDE libraries
+
    Copyright (C) 2007 Daniel Laidig <d.laidig@gmx.de>
- 
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
- 
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
- 
+
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
- 
+
    This file is autogenerated by kdeutils/kcharselect/kcharselect-generate-datafile.py
 */\n\n""")
     for group in data:
         for entry in group[1]:
-            out.write("I18N_NOOP2(\""+group[0]+"\", \""+entry+"\");\n")
- 
+            out.write(b"I18N_NOOP2(\""+group[0].encode('utf-8')+b"\", \""+entry.encode('utf-8')+b"\");\n")
+
 out = open("kcharselect-data", "wb")
 outTranslationDummy = open("kcharselect-translation.cpp", "wb")
- 
+
 inUnicodeData = open("UnicodeData.txt", "r")
 inNamesList = open("NamesList.txt", "r")
 inBlocks = open("Blocks.txt", "r")
-inSections = StringIO.StringIO(sectiondata)
+inSections = io.StringIO(sectiondata)
 inUnihan = open("Unihan_Readings.txt", "r")
- 
+
 if calcsize('=H') != 2 or calcsize('=I') != 4:
-    print "Error: Sizes of ushort and uint are not 16 and 32 bit as expected"
+    print("Error: Sizes of ushort and uint are not 16 and 32 bit as expected")
     sys.exit(1)
- 
+
 names = Names()
 details = Details()
 sectionsBlocks = SectionsBlocks()
 unihan = Unihan()
- 
+
 parser = Parser()
- 
-print "========== parsing files ==================="
+
+print("========== parsing files ===================")
 parser.parseUnicodeData(inUnicodeData, names)
-print ".",; sys.stdout.flush()
+print(".", end=' ')
+sys.stdout.flush()
 parser.parseDetails(inNamesList, details)
-print "\b.",; sys.stdout.flush()
+print("\b.", end=' ')
+sys.stdout.flush()
 parser.parseBlocks(inBlocks, sectionsBlocks)
-print "\b.",; sys.stdout.flush()
+print("\b.", end=' ')
+sys.stdout.flush()
 parser.parseSections(inSections, sectionsBlocks)
-print "\b.",; sys.stdout.flush()
+print("\b.", end=' ')
+sys.stdout.flush()
 parser.parseUnihan(inUnihan, unihan)
-print "\b.",; sys.stdout.flush()
- 
-print "done."
- 
+print("\b.", end=' ')
+sys.stdout.flush()
+
+print("done.")
+
 pos = 0
- 
+
 #write header, size: 40 bytes
-print "========== writing header =================="
+print("========== writing header ==================")
 out.write(pack("=I", 40))
-print "names strings begin", 40
- 
+print("names strings begin", 40)
+
 namesOffsetBegin = names.calculateStringSize() + 40
 out.write(pack("=I", namesOffsetBegin))
-print "names offsets begin", namesOffsetBegin
- 
+print("names offsets begin", namesOffsetBegin)
+
 detailsStringBegin = namesOffsetBegin + names.calculateOffsetSize()
 out.write(pack("=I", detailsStringBegin))
-print "details strings begin", detailsStringBegin
- 
+print("details strings begin", detailsStringBegin)
+
 detailsOffsetBegin = detailsStringBegin + details.calculateStringSize()
 out.write(pack("=I", detailsOffsetBegin))
-print "details offsets begin", detailsOffsetBegin
- 
+print("details offsets begin", detailsOffsetBegin)
+
 blocksStringBegin = detailsOffsetBegin + details.calculateOffsetSize()
 out.write(pack("=I", blocksStringBegin))
-print "block strings begin", blocksStringBegin
- 
+print("block strings begin", blocksStringBegin)
+
 blocksOffsetBegin = blocksStringBegin + sectionsBlocks.calculateBlockStringSize()
 out.write(pack("=I", blocksOffsetBegin))
-print "block offsets begin", blocksOffsetBegin
- 
+print("block offsets begin", blocksOffsetBegin)
+
 sectionStringBegin = blocksOffsetBegin + sectionsBlocks.calculateBlockOffsetSize()
 out.write(pack("=I", sectionStringBegin))
-print "section strings begin", sectionStringBegin
- 
+print("section strings begin", sectionStringBegin)
+
 sectionOffsetBegin = sectionStringBegin + sectionsBlocks.calculateSectionStringSize()
 out.write(pack("=I", sectionOffsetBegin))
-print "section offsets begin", sectionOffsetBegin
- 
+print("section offsets begin", sectionOffsetBegin)
+
 unihanStringBegin = sectionOffsetBegin + sectionsBlocks.calculateSectionOffsetSize()
 out.write(pack("=I", unihanStringBegin))
-print "unihan strings begin", unihanStringBegin
- 
+print("unihan strings begin", unihanStringBegin)
+
 unihanOffsetBegin = unihanStringBegin + unihan.calculateStringSize()
 out.write(pack("=I", unihanOffsetBegin))
-print "unihan offsets begin", unihanOffsetBegin
- 
+print("unihan offsets begin", unihanOffsetBegin)
+
 end = unihanOffsetBegin + unihan.calculateOffsetSize()
-print "end should be", end
- 
+print("end should be", end)
+
 pos += 40
- 
-print "========== writing data ===================="
- 
+
+print("========== writing data ====================")
+
 pos = names.writeStrings(out, pos)
-print "names strings written, position", pos
+print("names strings written, position", pos)
 pos = names.writeOffsets(out, pos)
-print "names offsets written, position", pos
+print("names offsets written, position", pos)
 pos = details.writeStrings(out, pos)
-print "details strings written, position", pos
+print("details strings written, position", pos)
 pos = details.writeOffsets(out, pos)
-print "details offsets written, position", pos
+print("details offsets written, position", pos)
 pos = sectionsBlocks.writeBlockStrings(out, pos)
-print sectionsBlocks.sections
-print "block strings written, position", pos
+print(sectionsBlocks.sections)
+print("block strings written, position", pos)
 pos = sectionsBlocks.writeBlockOffsets(out, pos)
-print "block offsets written, position", pos
+print("block offsets written, position", pos)
 pos = sectionsBlocks.writeSectionStrings(out, pos)
-print "section strings written, position", pos
+print("section strings written, position", pos)
 pos = sectionsBlocks.writeSectionOffsets(out, pos)
-print "section offsets written, position", pos
+print("section offsets written, position", pos)
 pos = unihan.writeStrings(out, pos)
-print "unihan strings written, position", pos
+print("unihan strings written, position", pos)
 pos = unihan.writeOffsets(out, pos)
-print "unihan offsets written, position", pos
- 
-print "========== writing translation dummy  ======"
+print("unihan offsets written, position", pos)
+
+print("========== writing translation dummy  ======")
 translationData = [["KCharSelect section name", sectionsBlocks.getSectionList()], ["KCharselect unicode block name",sectionsBlocks.getBlockList()]]
 writeTranslationDummy(outTranslationDummy, translationData)
-print "done. make sure to copy both kcharselect-data and kcharselect-translation.cpp."
-  
+print("done. make sure to copy both kcharselect-data and kcharselect-translation.cpp.")
