@@ -22,7 +22,8 @@
 #include "fcitx/instance.h"
 #include "fcitx-config/xdg.h"
 #include "fcitx/ime.h"
-#include "x11stuff.h"
+#include <setjmp.h>
+#include "xerrorhandler.h"
 
 static XErrorHandler   oldXErrorHandler;
 static XIOErrorHandler oldXIOErrorHandler;
@@ -38,18 +39,41 @@ void InitXErrorHandler(FcitxX11* x11priv)
     oldXIOErrorHandler = XSetIOErrorHandler(FcitxXIOErrorHandler);
 }
 
+void UnsetXErrorHandler(FcitxX11* x11priv)
+{
+    FCITX_UNUSED(x11priv);
+    x11handle = NULL;
+    /* we don't set back to old handler, to avoid any x error  */
+}
+
+extern jmp_buf FcitxRecover;
+
 int FcitxXIOErrorHandler(Display *d)
 {
+    FCITX_UNUSED(d);
+    if (!x11handle)
+        return 0;
+
+    if (FcitxInstanceGetIsDestroying(x11handle->owner))
+        return 0;
+
     /* Do not log, because this is likely to happen while log out */
     FcitxInstanceSaveAllIM(x11handle->owner);
 
     FcitxInstanceEnd(x11handle->owner);
-    return 0;
 
+    longjmp(FcitxRecover, 1);
+    return 0;
 }
 
 int FcitxXErrorHandler(Display * dpy, XErrorEvent * event)
 {
+    if (!x11handle)
+        return 0;
+
+    if (FcitxInstanceGetIsDestroying(x11handle->owner))
+        return 0;
+
     char    str[256];
     FILE* fp = NULL;
 
@@ -63,7 +87,8 @@ int FcitxXErrorHandler(Display * dpy, XErrorEvent * event)
 
     if (fp)
         fclose(fp);
-    if (event->error_code != 3 && event->error_code != BadMatch) {  // xterm will generate 3
+    if (event->error_code != 3 && event->error_code != BadMatch) {
+        // xterm will generate 3
         FcitxInstanceEnd(x11handle->owner);
     }
 
