@@ -42,9 +42,9 @@
 #include "MainWindow.h"
 #include "TrayWindow.h"
 #include "MenuWindow.h"
-#include "MessageWindow.h"
 #include "fcitx/hook.h"
 #include "fcitx-utils/utils.h"
+#include "module/notificationitem/fcitx-notificationitem.h"
 
 struct _FcitxSkin;
 static boolean MainMenuAction(FcitxUIMenu* menu, int index);
@@ -55,6 +55,7 @@ static void ClassicUICloseInputWindow(void* arg);
 static void ClassicUIShowInputWindow(void* arg);
 static void ClassicUIMoveInputWindow(void* arg);
 static void ClassicUIRegisterMenu(void *arg, FcitxUIMenu* menu);
+static void ClassicUIUnRegisterMenu(void *arg, FcitxUIMenu* menu);
 static void ClassicUIUpdateStatus(void *arg, FcitxUIStatus* status);
 static void ClassicUIRegisterStatus(void *arg, FcitxUIStatus* status);
 static void ClassicUIUpdateComplexStatus(void *arg, FcitxUIComplexStatus* status);
@@ -63,11 +64,13 @@ static void ClassicUIOnInputFocus(void *arg);
 static void ClassicUIOnInputUnFocus(void *arg);
 static void ClassicUIOnTriggerOn(void *arg);
 static void ClassicUIOnTriggerOff(void *arg);
-static void ClassicUIDisplayMessage(void *arg, char *title, char **msg, int length);
 static void ClassicUIInputReset(void *arg);
 static void ReloadConfigClassicUI(void *arg);
 static void ClassicUISuspend(void *arg);
 static void ClassicUIResume(void *arg);
+static void ClassicUIDelayedInitTray(void* arg);
+static void ClassicUIDelayedShowTray(void* arg);
+static void ClassicUINotificationItemAvailable(void* arg, boolean avaiable);
 
 static FcitxConfigFileDesc* GetClassicUIDesc();
 static void ClassicUIMainWindowSizeHint(void *arg, int* x, int* y,
@@ -87,7 +90,7 @@ FCITX_DEFINE_PLUGIN(fcitx_classic_ui, ui, FcitxUI) = {
     ClassicUIOnInputUnFocus,
     ClassicUIOnTriggerOn,
     ClassicUIOnTriggerOff,
-    ClassicUIDisplayMessage,
+    NULL,
     ClassicUIMainWindowSizeHint,
     ReloadConfigClassicUI,
     ClassicUISuspend,
@@ -95,7 +98,7 @@ FCITX_DEFINE_PLUGIN(fcitx_classic_ui, ui, FcitxUI) = {
     NULL,
     ClassicUIRegisterComplexStatus,
     ClassicUIUpdateComplexStatus,
-    NULL
+    ClassicUIUnRegisterMenu,
 };
 
 void* ClassicUICreate(FcitxInstance* instance)
@@ -143,11 +146,10 @@ void* ClassicUICreate(FcitxInstance* instance)
     classicui->mainMenu.priv = classicui;
     classicui->mainMenu.mark = -1;
 
-    classicui->inputWindow = CreateInputWindow(classicui);
-    classicui->mainWindow = CreateMainWindow(classicui);
-    classicui->trayWindow = CreateTrayWindow(classicui);
-    classicui->messageWindow = CreateMessageWindow(classicui);
-    classicui->mainMenuWindow = CreateMainMenuWindow(classicui);
+    classicui->inputWindow = InputWindowCreate(classicui);
+    classicui->mainWindow = MainWindowCreate(classicui);
+    classicui->trayWindow = TrayWindowCreate(classicui);
+    classicui->mainMenuWindow = MainMenuWindowCreate(classicui);
 
     FcitxIMEventHook resethk;
     resethk.arg = classicui;
@@ -157,9 +159,43 @@ void* ClassicUICreate(FcitxInstance* instance)
     DisplaySkin(classicui, classicui->skinType);
 
     FcitxClassicUIAddFunctions(instance);
+
+    classicui->waitDelayed = FcitxInstanceAddTimeout(instance, 0, ClassicUIDelayedInitTray, classicui);
+
     return classicui;
 }
 
+void ClassicUIDelayedInitTray(void* arg) {
+    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
+    // FcitxLog(INFO, "yeah we delayed!");
+    if (!classicui->bUseTrayIcon)
+        return;
+    /*
+     * if this return false, something wrong happened and callback
+     * will never be called, show tray directly
+     */
+    if (FcitxNotificationItemEnable(classicui->owner, ClassicUINotificationItemAvailable, classicui)) {
+        if (!classicui->trayTimeout)
+            classicui->trayTimeout = FcitxInstanceAddTimeout(classicui->owner, 100, ClassicUIDelayedShowTray, classicui);
+    } else {
+        TrayWindowRelease(classicui->trayWindow);
+        TrayWindowInit(classicui->trayWindow);
+    }
+}
+
+void ClassicUIDelayedShowTray(void* arg)
+{
+    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
+    classicui->trayTimeout = 0;
+    if (!classicui->bUseTrayIcon)
+        return;
+
+    if (!classicui->trayWindow->bTrayMapped) {
+        TrayWindowRelease(classicui->trayWindow);
+        TrayWindowInit(classicui->trayWindow);
+    }
+}
+
 void ClassicUISetWindowProperty(FcitxClassicUI* classicui, Window window, FcitxXWindowType type, char *windowTitle)
 {
     FcitxX11SetWindowProp(classicui->owner, &window, &type, windowTitle);
@@ -170,38 +206,39 @@ static void ClassicUIInputReset(void *arg)
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     if (classicui->isSuspend)
         return;
-    DrawMainWindow(classicui->mainWindow);
-    DrawTrayWindow(classicui->trayWindow);
+    MainWindowShow(classicui->mainWindow);
+    TrayWindowDraw(classicui->trayWindow);
+    classicui->inputWindow->highlight = 0;
 }
 
 static void ClassicUICloseInputWindow(void *arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    CloseInputWindowInternal(classicui->inputWindow);
+    InputWindowClose(classicui->inputWindow);
 }
 
 static void ClassicUIShowInputWindow(void *arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    ShowInputWindowInternal(classicui->inputWindow);
+    InputWindowShow(classicui->inputWindow);
 }
 
 static void ClassicUIMoveInputWindow(void *arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    MoveInputWindowInternal(classicui->inputWindow);
+    classicui->inputWindow->parent.MoveWindow(&classicui->inputWindow->parent);
 }
 
 static void ClassicUIUpdateStatus(void *arg, FcitxUIStatus* status)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    DrawMainWindow(classicui->mainWindow);
+    MainWindowShow(classicui->mainWindow);
 }
 
 void ClassicUIUpdateComplexStatus(void* arg, FcitxUIComplexStatus* status)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    DrawMainWindow(classicui->mainWindow);
+    MainWindowShow(classicui->mainWindow);
 }
 
 void ClassicUIRegisterComplexStatus(void* arg, FcitxUIComplexStatus* status)
@@ -214,11 +251,17 @@ void ClassicUIRegisterComplexStatus(void* arg, FcitxUIComplexStatus* status)
 static void ClassicUIRegisterMenu(void *arg, FcitxUIMenu* menu)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    XlibMenu* xlibMenu = CreateXlibMenu(classicui);
+    XlibMenu* xlibMenu = XlibMenuCreate(classicui);
     menu->uipriv[classicui->isfallback] = xlibMenu;
     xlibMenu->menushell = menu;
 }
 
+static void ClassicUIUnRegisterMenu(void *arg, FcitxUIMenu* menu)
+{
+    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
+    XlibMenuDestroy((XlibMenu*) menu->uipriv[classicui->isfallback]);
+}
+
 static void ClassicUIRegisterStatus(void *arg, FcitxUIStatus* status)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
@@ -240,8 +283,8 @@ static void ClassicUIOnInputFocus(void *arg)
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     if (classicui->isSuspend)
         return;
-    DrawMainWindow(classicui->mainWindow);
-    DrawTrayWindow(classicui->trayWindow);
+    MainWindowShow(classicui->mainWindow);
+    TrayWindowDraw(classicui->trayWindow);
 }
 
 static void ClassicUIOnInputUnFocus(void *arg)
@@ -249,24 +292,45 @@ static void ClassicUIOnInputUnFocus(void *arg)
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     if (classicui->isSuspend)
         return;
-    DrawMainWindow(classicui->mainWindow);
-    DrawTrayWindow(classicui->trayWindow);
+    MainWindowShow(classicui->mainWindow);
+    TrayWindowDraw(classicui->trayWindow);
 }
 
 void ClassicUISuspend(void* arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     classicui->isSuspend = true;
-    CloseInputWindowInternal(classicui->inputWindow);
-    CloseMainWindow(classicui->mainWindow);
-    ReleaseTrayWindow(classicui->trayWindow);
+    classicui->notificationItemAvailable = false;
+    InputWindowClose(classicui->inputWindow);
+    MainWindowClose(classicui->mainWindow);
+    TrayWindowRelease(classicui->trayWindow);
+    /* always call this function will not do anything harm */
+    FcitxNotificationItemDisable(classicui->owner);
 }
 
 void ClassicUIResume(void* arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
     classicui->isSuspend = false;
-    InitTrayWindow(classicui->trayWindow);
+    ClassicUIDelayedInitTray(classicui);
+}
+
+void ClassicUINotificationItemAvailable(void* arg, boolean avaiable) {
+    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
+    /* ClassicUISuspend has already done all clean up */
+    if (classicui->isSuspend)
+        return;
+    classicui->notificationItemAvailable = avaiable;
+    if (!avaiable) {
+        TrayWindowRelease(classicui->trayWindow);
+        TrayWindowInit(classicui->trayWindow);
+    } else {
+        if (classicui->trayTimeout) {
+            FcitxInstanceRemoveTimeoutById(classicui->owner, classicui->trayTimeout);
+            classicui->trayTimeout = 0;
+        }
+        TrayWindowRelease(classicui->trayWindow);
+    }
 }
 
 void ActivateWindow(Display *dpy, int iScreen, Window window)
@@ -291,11 +355,6 @@ void ActivateWindow(Display *dpy, int iScreen, Window window)
     XSync(dpy, False);
 }
 
-void GetScreenSize(FcitxClassicUI* classicui, int* width, int* height)
-{
-    FcitxX11GetScreenSize(classicui->owner, width, height);
-}
-
 FcitxRect GetScreenGeometry(FcitxClassicUI* classicui, int x, int y)
 {
     FcitxRect result = { 0, 0 , 0 , 0 };
@@ -311,10 +370,7 @@ boolean LoadClassicUIConfig(FcitxClassicUI* classicui)
     if (configDesc == NULL)
         return false;
     FILE *fp;
-    char *file;
-    fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "r", &file);
-    FcitxLog(DEBUG, "Load Config File %s", file);
-    free(file);
+    fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "r", NULL);
     if (!fp) {
         if (errno == ENOENT)
             SaveClassicUIConfig(classicui);
@@ -333,11 +389,8 @@ boolean LoadClassicUIConfig(FcitxClassicUI* classicui)
 void SaveClassicUIConfig(FcitxClassicUI *classicui)
 {
     FcitxConfigFileDesc* configDesc = GetClassicUIDesc();
-    char *file;
-    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "w", &file);
-    FcitxLog(DEBUG, "Save Config to %s", file);
+    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "w", NULL);
     FcitxConfigSaveConfigFileFp(fp, &classicui->gconfig, configDesc);
-    free(file);
     if (fp)
         fclose(fp);
 }
@@ -358,28 +411,17 @@ ClassicUIMouseClick(FcitxClassicUI* classicui, Window window, int *x, int *y)
 void ClassicUIOnTriggerOn(void* arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    FcitxInstance *instance = classicui->owner;
-    if (FcitxInstanceGetCurrentStatev2(instance) == IS_ACTIVE) {
-        DrawMainWindow(classicui->mainWindow);
-    }
-    DrawTrayWindow(classicui->trayWindow);
+    MainWindowShow(classicui->mainWindow);
+    TrayWindowDraw(classicui->trayWindow);
 }
 
 void ClassicUIOnTriggerOff(void* arg)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    DrawMainWindow(classicui->mainWindow);
-    DrawTrayWindow(classicui->trayWindow);
+    MainWindowShow(classicui->mainWindow);
+    TrayWindowDraw(classicui->trayWindow);
 }
 
-void ClassicUIDisplayMessage(void* arg, char* title, char** msg, int length)
-{
-    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
-    XMapRaised(classicui->dpy, classicui->messageWindow->window);
-    DrawMessageWindow(classicui->messageWindow, title, msg, length);
-}
-
-
 static void UpdateMainMenu(FcitxUIMenu* menu)
 {
     FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
@@ -397,7 +439,7 @@ static void UpdateMainMenu(FcitxUIMenu* menu)
             status = (FcitxUIStatus*) utarray_next(uistats, status)
         ) {
         FcitxClassicUIStatus* privstat =  GetPrivateStatus(status);
-        if (privstat == NULL || !status->visible || privstat->avail )
+        if (privstat == NULL || !status->visible)
             continue;
 
         flag = true;
@@ -411,7 +453,7 @@ static void UpdateMainMenu(FcitxUIMenu* menu)
             compstatus = (FcitxUIComplexStatus*) utarray_next(uicompstats, compstatus)
         ) {
         FcitxClassicUIStatus* privstat =  GetPrivateStatus(compstatus);
-        if (privstat == NULL || !compstatus->visible || privstat->avail)
+        if (privstat == NULL || !compstatus->visible)
             continue;
         if (FcitxUIGetMenuByStatusName(instance, compstatus->name))
             continue;
@@ -441,8 +483,6 @@ static void UpdateMainMenu(FcitxUIMenu* menu)
             if (compStatus) {
                 if (!compStatus->visible)
                     continue;
-                if (GetPrivateStatus(compStatus)->avail)
-                    continue;
             }
         }
 
@@ -491,15 +531,6 @@ boolean MainMenuAction(FcitxUIMenu* menu, int index)
     return true;
 }
 
-void
-ClassicUIInitWindowAttribute(FcitxClassicUI *classicui, Visual **vs,
-                             Colormap *cmap, XSetWindowAttributes *attrib,
-                             unsigned long *attribmask, int *depth)
-{
-    FcitxX11InitWindowAttribute(classicui->owner, vs, cmap, attrib,
-                                attribmask, depth);
-}
-
 Visual * ClassicUIFindARGBVisual(FcitxClassicUI* classicui)
 {
     return FcitxX11FindARGBVisual(classicui->owner);
@@ -516,7 +547,7 @@ void ClassicUIMainWindowSizeHint(void* arg, int* x, int* y, int* w, int* h)
     }
 
     XWindowAttributes attr;
-    XGetWindowAttributes(classicui->dpy, classicui->mainWindow->window, &attr);
+    XGetWindowAttributes(classicui->dpy, classicui->mainWindow->parent.wId, &attr);
     if (w) {
         *w = attr.width;
     }
